@@ -2,6 +2,7 @@
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
  * Copyright (C) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 2001, 2002 The ProFTPD Project team
  *  
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +27,7 @@
 /*
  * Configuration structure, server, command and associated prototypes.
  *
- * $Id: dirtree.h,v 1.15 2002/05/21 20:47:15 castaglia Exp $
+ * $Id: dirtree.h,v 1.19 2002/08/01 22:08:54 castaglia Exp $
  */
 
 #ifndef __DIRTREE_H
@@ -188,8 +189,11 @@ extern int			TimeoutStalled;
 void kludge_disable_umask(void);
 void kludge_enable_umask(void);
 
+int define_exists(const char *);
 void init_config(void);
 void fixup_servers(void);
+void set_config_stream(FILE *, unsigned int);
+char *get_config_line(char *, size_t);
 int parse_config_file(const char *);
 config_rec *add_config_set(xaset_t **, const char *);
 config_rec *add_config(const char *);
@@ -219,6 +223,7 @@ server_rec *end_new_server(void);
 config_rec *start_sub_config(const char *);
 config_rec *end_sub_config(void);
 char *get_word(char **);
+char *get_line(char *, int, FILE *, unsigned int *);
 
 config_rec *dir_match_path(pool *, char *);
 void build_dyn_config(pool *, char *, struct stat *, int);
