@@ -1,7 +1,8 @@
 /*
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
- * Copyright (C) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 2001, 2002 The ProFTPD Project team
  *  
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +26,7 @@
 
 /* BSD socket manipulation tools.
  *
- * $Id: inet.h,v 1.8 2002/05/21 20:47:15 castaglia Exp $
+ * $Id: inet.h,v 1.12 2002/08/14 16:17:35 castaglia Exp $
  */
 
 #ifndef __INET_H
@@ -93,6 +94,7 @@ conn_t *inet_create_connection(pool *, xaset_t *, int, p_in_addr_t *, int, int);
 conn_t *inet_create_connection_portrange(pool *, xaset_t *, p_in_addr_t *,
   int, int);
 void inet_close(pool *, conn_t *);
+void inet_lingering_close(pool *, conn_t *, long);
 int inet_setnonblock(pool *, conn_t *);
 int inet_setblock(pool *, conn_t *);
 int inet_setoptions(pool *, conn_t *, int, int);
@@ -104,7 +106,7 @@ int inet_accept_nowait(pool *, conn_t *);
 int inet_connect(pool *, conn_t *, p_in_addr_t *, int);
 int inet_connect_nowait(pool*,conn_t*,p_in_addr_t*,int);
 int inet_get_conn_info(conn_t *, int);
-conn_t *inet_accept(pool *, conn_t *, conn_t *, int, int, int);
+conn_t *inet_accept(pool *, conn_t *, conn_t *, int, int, unsigned char);
 conn_t *inet_associate(pool *, conn_t *, p_in_addr_t *, IOFILE *,
   IOFILE *, int);
 conn_t *inet_openrw(pool *, conn_t *, p_in_addr_t *, int, int, int, int);
