@@ -1,7 +1,8 @@
 /*
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
- * Copyright (C) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 2001, 2002 The ProFTPD Project team
  *  
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +25,7 @@
  */
 
 /* General options
- * $Id: proftpd.h,v 1.21 2002/05/21 20:47:15 castaglia Exp $
+ * $Id: proftpd.h,v 1.29 2002/08/14 16:17:35 castaglia Exp $
  */
 
 #ifndef __PROFTPD_H
@@ -149,7 +150,7 @@ typedef struct {
   struct config_struc *anon_config;	/* Anonymous FTP configuration */
   char *anon_user;			/* E-mail address sent to us */
   
-  unsigned long restart_pos;		/* Restart marked position */
+  off_t restart_pos;			/* Restart marked position */
 
   struct {
     struct pool *p;
@@ -164,15 +165,15 @@ typedef struct {
     int bufsize,buflen;
 
     struct timeval start_time;		/* Time current transfer started */
-    unsigned long file_size;		/* Total size of file (if known) */
-    unsigned long total_bytes;		/* Total bytes transfered */
+    off_t file_size;			/* Total size of file (if known) */
+    off_t total_bytes;			/* Total bytes transfered */
 
     int (*get_data)(struct IO_File*,char*,int);
     int (*complete)(struct IO_File*);
     int (*abort)(struct IO_File*,int err);
   } xfer;
 
-  unsigned long total_bytes;          /* Total bytes transfered for this session */
+  off_t total_bytes;          /* Total bytes transfered for this session */
 
 } session_t;
 
@@ -185,6 +186,7 @@ extern array_header *daemon_gids;
 #define STOR_DEFAULT	0
 #define STOR_APPEND	1
 #define STOR_HIDDEN	2
+#define STOR_UNIQUE	3
 
 extern session_t	session;
 extern char		ServerType;
@@ -212,11 +214,25 @@ extern const char	*pwdfname,*grpfname;
 #define SERVER_INETD		0
 #define SERVER_STANDALONE	1
 
+/* Signals */
+#define RECEIVED_SIG_REHASH	0x0001
+#define RECEIVED_SIG_EXIT	0x0002
+#define RECEIVED_SIG_SHUTDOWN	0x0004
+#define RECEIVED_SIG_SEGV	0x0008
+#define RECEIVED_SIG_TERMINATE	0x0010
+#define RECEIVED_SIG_XCPU	0x0020
+#define RECEIVED_SIG_TERM_OTHER	0x0040
+#define RECEIVED_SIG_ABORT	0x0080
+#define RECEIVED_SIG_DEBUG	0x0100
+#define RECEIVED_SIG_CHLD	0x0200
+#define RECEIVED_SIG_ALRM	0x0400
+
 /* Timers */
 #define TIMER_LOGIN		1
 #define TIMER_IDLE		2
 #define TIMER_NOXFER		3
 #define TIMER_STALLED		4
+#define TIMER_SESSION		5
 
 /* Response structure */
 
@@ -229,15 +245,68 @@ typedef struct response_struc {
 
 /* Misc Prototypes */
 
-void send_response_raw(const char *, ...);
-void send_response_async(const char *, const char *, ...);
-void send_response(const char *, const char *, ...);
-void send_response_ml_start(const char *, const char *, ...);
-void send_response_ml(const char *, ...);
-void send_response_ml_end(const char *, ...);
-void add_response(const char*, const char *, ...);
-void add_response_err(const char *, const char *, ...);
+/* Utilize gcc's __attribute__ pragma for signalling that it should perform
+ * printf-style checking of this function's arguments.
+ */
+
+void send_response_raw(const char *, ...)
+#ifdef __GNUC__
+       __attribute__ ((format (printf, 1, 2)));
+#else
+       ;
+#endif
+
+void send_response_async(const char *, const char *, ...)
+#ifdef __GNUC__
+       __attribute__ ((format (printf, 2, 3)));
+#else
+       ;
+#endif
+
+void send_response(const char *, const char *, ...)
+#ifdef __GNUC__
+       __attribute__ ((format (printf, 2, 3)));
+#else
+       ;
+#endif
+
+void send_response_ml_start(const char *, const char *, ...)
+#ifdef __GNUC__
+       __attribute__ ((format (printf, 2, 3)));
+#else
+       ;
+#endif
+
+void send_response_ml(const char *, ...)
+#ifdef __GNUC__
+       __attribute__ ((format (printf, 1, 2)));
+#else
+       ;
+#endif
+
+void send_response_ml_end(const char *, ...)
+#ifdef __GNUC__
+       __attribute__ ((format (printf, 1, 2)));
+#else
+       ;
+#endif
+
+void add_response(const char *, const char *, ...)
+#ifdef __GNUC__
+       __attribute__ ((format (printf, 2, 3)));
+#else
+       ;
+#endif
+
+void add_response_err(const char *, const char *, ...)
+#ifdef __GNUC__
+       __attribute__ ((format (printf, 2, 3)));
+#else
+       ;
+#endif
+
 void end_login(int);
+void pr_handle_signals(void);
 void main_exit(void *, void *, void *, void *);
 void main_set_idle(void);
 void register_rehash(void *, void(*)(void *));
