@@ -1,7 +1,8 @@
 /*
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
- * Copyright (C) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 2001, 2002 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +25,7 @@
  */
 
 /* Non-specific support functions.
- * $Id: support.h,v 1.8 2002/05/21 20:47:15 castaglia Exp $
+ * $Id: support.h,v 1.11 2002/07/09 22:20:10 castaglia Exp $
  */
 
 #ifndef __SUPPORT_H
@@ -76,7 +77,6 @@ void run_exit_handlers(void);
 void schedule(void (*f)(void *, void *, void *, void *), int, void *, void *,
   void *, void *);
 void run_schedule(void);
-int schedulep(void);
 
 int get_name_max(char *, int);
 
@@ -97,6 +97,6 @@ char *sstrncpy(char *, const char *, size_t);
 char *sreplace(pool *, char *, ...);
 
 #if defined(HAVE_SYS_STATVFS_H) || defined(HAVE_SYS_VFS_H)
-unsigned long get_fs_size(char *);
+off_t get_fs_size(char *);
 #endif
 #endif /* __SUPPORT_H */
