@@ -1,7 +1,8 @@
 /*
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
- * Copyright (C) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 2001, 2002 The ProFTPD Project team
  *  
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +26,7 @@
 
 /*
  * Authentication module for ProFTPD
- * $Id: mod_auth.c,v 1.72 2002/05/21 20:47:16 castaglia Exp $
+ * $Id: mod_auth.c,v 1.80 2002/07/22 23:18:50 castaglia Exp $
  */
 
 #include "conf.h"
@@ -39,6 +40,7 @@ module auth_module;
 
 static int logged_in = 0;
 static int auth_tries = 0;
+static unsigned int TimeoutSession = 0;
 
 static void _do_user_counts(void);
 
@@ -100,8 +102,20 @@ int _login_timeout(CALLBACK_FRAME)
   main_exit((void*) LOG_NOTICE, "FTP login timed out, disconnected.",
 	    (void*) 0, NULL);
   
-/* should never be reached */
-  return 0;		/* Don't restart the timer */
+  /* Do not restart the timer (should never be reached). */
+  return 0;
+}
+
+static int session_timeout(CALLBACK_FRAME) {
+
+  send_response_async(R_421, "Session Timeout (%u seconds): "
+    "closing control connection", TimeoutSession);
+
+  main_exit((void *) LOG_NOTICE, "FTP session timed out, disconnected",
+    (void *) 0, NULL);
+
+  /* no need to restart the timer -- session's over */
+  return 0;
 }
 
 static int auth_child_init(void) {
@@ -156,12 +170,11 @@ static int auth_init(void) {
   return 0;
 }
 
-static int _do_auth(pool *p, xaset_t *conf, char *u, char *pw)
-{
+static int _do_auth(pool *p, xaset_t *conf, char *u, char *pw) {
   char *cpw = NULL;
   config_rec *c;
 
-  if(conf) {
+  if (conf) {
     c = find_config(conf,CONF_PARAM,"UserPassword",FALSE);
 
     while(c) {
@@ -174,16 +187,111 @@ static int _do_auth(pool *p, xaset_t *conf, char *u, char *pw)
     }
   }
 
-
-  if(cpw) {
-    if(!auth_getpwnam(p,u))
+  if (cpw) {
+    if (!auth_getpwnam(p, u))
       return AUTH_NOPWD;
-    return auth_check(p,cpw,u,pw);
+
+    return auth_check(p, cpw, u, pw);
   }
 
   return auth_authenticate(p,u,pw);
 }
 
+MODRET post_cmd_pass(cmd_rec *cmd) {
+  config_rec *c = NULL;
+  unsigned int ctxt_precedence = 0;
+  unsigned char have_user_timeout, have_group_timeout, have_class_timeout,
+    have_all_timeout;
+
+  have_user_timeout = have_group_timeout = have_class_timeout =
+    have_all_timeout = FALSE;
+
+  if ((c = find_config(CURRENT_CONF, CONF_PARAM, "TimeoutSession",
+      FALSE)) != NULL) {
+
+    if (c->argc == 3) {
+      if (!strcmp(c->argv[1], "user")) {
+        if (user_expression((char **) &c->argv[2])) {
+
+          if (*((unsigned int *) c->argv[1]) > ctxt_precedence) {
+
+            /* Set the context precedence. */
+            ctxt_precedence = *((unsigned int *) c->argv[1]);
+
+            TimeoutSession = *((unsigned int *) c->argv[0]);
+
+            have_group_timeout = have_class_timeout = have_all_timeout = FALSE;
+            have_user_timeout = TRUE;
+          }
+        }
+
+      } else if (!strcmp(c->argv[1], "group")) {
+        if (group_expression((char **) &c->argv[2])) {
+
+          if (*((unsigned int *) c->argv[1]) > ctxt_precedence) {
+
+            /* Set the context precedence. */
+            ctxt_precedence = *((unsigned int *) c->argv[1]);
+
+            TimeoutSession = *((unsigned int *) c->argv[0]);
+
+            have_user_timeout = have_class_timeout = have_all_timeout = FALSE;
+            have_group_timeout = TRUE;
+          }
+        }
+
+      } else if (!strcmp(c->argv[1], "class")) {
+        if (session.class && session.class->name &&
+            !strcmp(session.class->name, c->argv[2])) {
+
+          if (*((unsigned int *) c->argv[1]) > ctxt_precedence) {
+
+            /* Set the context precedence. */
+            ctxt_precedence = *((unsigned int *) c->argv[1]);
+
+            TimeoutSession = *((unsigned int *) c->argv[0]);
+
+            have_user_timeout = have_group_timeout = have_all_timeout = FALSE;
+            have_class_timeout = TRUE;
+          }
+        }
+      }
+
+    } else {
+
+      if (*((unsigned int *) c->argv[1]) > ctxt_precedence) {
+
+        /* Set the context precedence. */
+        ctxt_precedence = *((unsigned int *) c->argv[1]);
+
+        TimeoutSession = *((unsigned int *) c->argv[0]);
+
+        have_user_timeout = have_group_timeout = have_class_timeout = FALSE;
+        have_all_timeout = TRUE;
+      }
+    }
+
+    c = find_config_next(c, c->next, CONF_PARAM, "TimeoutSession", FALSE);
+  }
+
+  /* If configured, start a session timer.  The timer ID value for
+   * session timers will not be #defined, as I think that is a bad approach.
+   * A better mechanism would be to use the random timer ID generation, and
+   * store the returned ID in order to later remove the timer.
+   */
+
+  if (have_user_timeout || have_group_timeout ||
+      have_class_timeout || have_all_timeout) {
+    log_debug(DEBUG4, "setting TimeoutSession of %u seconds for current %s",
+      TimeoutSession,
+      have_user_timeout ? "user" : have_group_timeout ? "group" :
+      have_class_timeout ? "class" : "all");
+    add_timer(TimeoutSession, TIMER_SESSION, &auth_module, session_timeout);
+  }
+
+  return HANDLED(cmd);
+}
+
 /* Handle group based authentication, only checked if pw
  * based fails
  */
@@ -1151,7 +1259,9 @@ static int _setup_environment(pool *p, char *user, char *pass)
   /* auth_endpwent(p); */
 
   /* Default transfer mode is ASCII */
-  defaulttransfermode = (char*)get_param_ptr(CURRENT_CONF, "DefaultTransferMode", FALSE);
+  defaulttransfermode = (char*)get_param_ptr(main_server->conf,
+    "DefaultTransferMode", FALSE);
+
   if (defaulttransfermode && strcasecmp(defaulttransfermode, "binary") == 0)
 	session.flags &= (SF_ALL^SF_ASCII);
   else
@@ -1606,14 +1716,16 @@ MODRET cmd_pass(cmd_rec *cmd) {
     if(display)
       core_display_file(R_230, display, NULL);
     
-    if((grantmsg = (char*)get_param_ptr((session.anon_config ?
+    if ((grantmsg = (char *)get_param_ptr((session.anon_config ?
         session.anon_config->subset : cmd->server->conf),
         "AccessGrantMsg",FALSE)) != NULL) {
       grantmsg = sreplace(cmd->tmp_pool, grantmsg, "%u", user, NULL);
 
-      add_response(R_230, "%s", grantmsg, NULL);
+      add_response(R_230, "%s", grantmsg);
+
     } else {
-      if(session.flags & SF_ANON)
+
+      if (session.flags & SF_ANON)
         add_response(R_230, "Anonymous access granted, restrictions apply.");
       else
         add_response(R_230, "User %s logged in.", user);
@@ -1641,11 +1753,11 @@ MODRET cmd_pass(cmd_rec *cmd) {
     if(max == -1)
       max = 3;
 
-    if(++auth_tries >= max) {
+    if (++auth_tries >= max) {
       if (denymsg)
-        send_response(R_530, "%s", denymsg, NULL);
+        send_response(R_530, "%s", denymsg);
       else
-      send_response(R_530,"Login incorrect");
+        send_response(R_530, "Login incorrect.");
 
       log_auth(LOG_NOTICE, "Maximum login attempts exceeded.");
       end_login(0);
@@ -1660,13 +1772,11 @@ MODRET cmd_pass(cmd_rec *cmd) {
   return HANDLED(cmd);
 }
 
-
 MODRET cmd_acct(cmd_rec *cmd) {
   add_response(R_502, "ACCT command not implemented.");
   return HANDLED(cmd);
 }
 
-
 MODRET cmd_rein(cmd_rec *cmd) {
   add_response(R_502, "REIN command not implemented.");
   return HANDLED(cmd);
@@ -1675,6 +1785,76 @@ MODRET cmd_rein(cmd_rec *cmd) {
 /* Configuration handlers
  */
 
+MODRET set_accessdenymsg(cmd_rec *cmd) {
+  config_rec *c = NULL;
+
+  CHECK_ARGS(cmd, 1);
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL|CONF_ANON);
+
+  c = add_config_param_str(cmd->argv[0], 1, cmd->argv[1]);
+  c->flags |= CF_MERGEDOWN;
+
+  return HANDLED(cmd);
+}
+
+MODRET set_accessgrantmsg(cmd_rec *cmd) {
+  config_rec *c = NULL;
+
+  CHECK_ARGS(cmd, 1);
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL|CONF_ANON);
+
+  c = add_config_param_str(cmd->argv[0], 1, cmd->argv[1]);
+  c->flags |= CF_MERGEDOWN;
+
+  return HANDLED(cmd);
+}
+
+MODRET set_anonrequirepassword(cmd_rec *cmd) {
+  int bool = -1;
+
+  CHECK_ARGS(cmd, 1);
+  CHECK_CONF(cmd, CONF_ANON);
+
+  if ((bool = get_boolean(cmd, 1)) == -1)
+    CONF_ERROR(cmd, "expected boolean parameter");
+
+  add_config_param(cmd->argv[0], 1, (void *) bool);
+
+  return HANDLED(cmd);
+}
+
+MODRET add_anonymousgroup(cmd_rec *cmd) {
+  int argc;
+  config_rec *c = NULL;
+  char **argv = NULL;
+  array_header *acl = NULL;
+
+  if (cmd->argc < 2)
+    CONF_ERROR(cmd, "wrong number of parameters");
+
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL);
+
+  argv = cmd->argv;
+  argc = cmd->argc - 1;
+
+  acl = parse_group_expression(cmd->tmp_pool, &argc, argv);
+
+  c = add_config_param(cmd->argv[0], 0);
+  c->argc = argc;
+  c->argv = pcalloc(c->pool,(argc+1) * sizeof(char*));
+  argv = (char **)c->argv;
+
+  if (argc && acl)
+    while (argc--) {
+      *argv++ = pstrdup(c->pool, *((char **) acl->elts));
+      acl->elts = ((char **) acl->elts) + 1;
+    }
+
+  *argv = NULL;
+
+  return HANDLED(cmd);
+}
+
 MODRET set_authaliasonly(cmd_rec *cmd) {
   int bool = -1;
   config_rec *c = NULL;
@@ -1689,41 +1869,21 @@ MODRET set_authaliasonly(cmd_rec *cmd) {
   c->argv[0] = pcalloc(c->pool, sizeof(int));
   *((int *) c->argv[0]) = bool;
 
+  c->flags |= CF_MERGEDOWN;
   return HANDLED(cmd);
 }
 
-MODRET set_rootlogin(cmd_rec *cmd) {
+MODRET set_authusingalias(cmd_rec *cmd) {
   int bool = -1;
-  config_rec *c = NULL;
 
-  CHECK_ARGS(cmd,1);
-  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL|CONF_ANON);
-  
-  if ((bool = get_boolean(cmd, 1)) == -1)
-    CONF_ERROR(cmd, "expected boolean argument.");
- 
-  c = add_config_param(cmd->argv[0], 1, NULL);
-  c->argv[0] = pcalloc(c->pool, sizeof(unsigned char));
-  *((unsigned char *) c->argv[0]) = (unsigned char) bool;
+  CHECK_ARGS(cmd, 1);
+  CHECK_CONF(cmd, CONF_ANON);
 
-  c->flags |= CF_MERGEDOWN;
+  if ((bool = get_boolean(cmd,1)) == -1)
+    CONF_ERROR(cmd, "expected boolean parameter");
 
-  return HANDLED(cmd);
-}
+  add_config_param(cmd->argv[0], 1, (void *) bool);
 
-MODRET set_loginpasswordprompt(cmd_rec *cmd) {
-  config_rec *c;
-  int bool;
-  
-  CHECK_ARGS(cmd,1);
-  CHECK_CONF(cmd,CONF_ROOT|CONF_VIRTUAL|CONF_ANON|CONF_GLOBAL);
- 
-  if ((bool = get_boolean(cmd, 1)) == -1)
-    CONF_ERROR(cmd, "expected boolean argument.");
-  
-  c = add_config_param("LoginPasswordPrompt", 1, (void*) bool);
-  c->flags |= CF_MERGEDOWN;
-  
   return HANDLED(cmd);
 }
 
@@ -1758,16 +1918,16 @@ MODRET add_defaultroot(cmd_rec *cmd) {
   
   acl = parse_group_expression(cmd->tmp_pool, &argc, argv);
   
-  c = add_config_param("DefaultRoot", 0);
+  c = add_config_param(cmd->argv[0], 0);
   
   c->argc = argc + 1;
   c->argv = pcalloc(c->pool, (argc + 2) * sizeof(char *));
   argv = (char **) c->argv;
-  *argv++ = pstrdup(permanent_pool, dir);
+  *argv++ = pstrdup(c->pool, dir);
   
   if(argc && acl)
     while(argc--) {
-      *argv++ = pstrdup(permanent_pool, *((char **) acl->elts));
+      *argv++ = pstrdup(c->pool, *((char **) acl->elts));
       acl->elts = ((char **) acl->elts) + 1;
     }
 
@@ -1781,7 +1941,7 @@ MODRET add_defaultchdir(cmd_rec *cmd) {
   int argc;
   array_header *acl = NULL;
 
-  CHECK_CONF(cmd, CONF_ROOT | CONF_VIRTUAL | CONF_ANON | CONF_GLOBAL);
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL|CONF_ANON);
   
   if(cmd->argc < 2)
     CONF_ERROR(cmd, "syntax: DefaultChdir <directory> [<group-expression>]");
@@ -1800,20 +1960,225 @@ MODRET add_defaultchdir(cmd_rec *cmd) {
   
   acl = parse_group_expression(cmd->tmp_pool, &argc, argv);
   
-  c = add_config_param("DefaultChdir", 0);
+  c = add_config_param(cmd->argv[0], 0);
   
   c->argc = argc + 1;
   c->argv = pcalloc(c->pool, (argc + 2) * sizeof(char *));
   argv = (char **) c->argv;
-  *argv++ = pstrdup(permanent_pool, dir);
+  *argv++ = pstrdup(c->pool, dir);
   
   if(argc && acl)
     while(argc--) {
-      *argv++ = pstrdup(permanent_pool, *((char **) acl->elts));
+      *argv++ = pstrdup(c->pool, *((char **) acl->elts));
       acl->elts = ((char **) acl->elts) + 1;
     }
   
   *argv = NULL;
+
+  c->flags |= CF_MERGEDOWN;
+  return HANDLED(cmd);
+}
+
+MODRET set_grouppassword(cmd_rec *cmd) {
+  config_rec *c = NULL;
+
+  CHECK_ARGS(cmd, 2);
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL|CONF_ANON);
+
+  c = add_config_param_str(cmd->argv[0], 2, cmd->argv[1], cmd->argv[2]);
+  c->flags |= CF_MERGEDOWN;
+
+  return HANDLED(cmd);
+}
+
+MODRET set_loginpasswordprompt(cmd_rec *cmd) {
+  int bool = -1;
+  config_rec *c = NULL;
+
+  CHECK_ARGS(cmd, 1);
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL|CONF_ANON);
+
+  if ((bool = get_boolean(cmd, 1)) == -1)
+    CONF_ERROR(cmd, "expected boolean argument.");
+
+  c = add_config_param(cmd->argv[0], 1, (void*) bool);
+
+  c->flags |= CF_MERGEDOWN;
+  return HANDLED(cmd);
+}
+
+MODRET set_maxloginattempts(cmd_rec *cmd) {
+  int max;
+
+  CHECK_ARGS(cmd, 1);
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL);
+
+  if (!strcasecmp(cmd->argv[1], "none"))
+    max = 0;
+
+  else {
+    char *endp = NULL;
+    max = (int) strtol(cmd->argv[1], &endp, 10);
+
+    if ((endp && *endp) || max < 1)
+      CONF_ERROR(cmd, "parameter must be 'none' or a number greater than 0");
+  }
+
+  add_config_param(cmd->argv[0], 1, (void*)max);
+  return HANDLED(cmd);
+}
+
+MODRET set_requirevalidshell(cmd_rec *cmd) {
+  int bool = -1;
+  config_rec *c = NULL;
+
+  CHECK_ARGS(cmd, 1);
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL|CONF_ANON);
+
+  if ((bool = get_boolean(cmd, 1)) == -1)
+    CONF_ERROR(cmd, "expected boolean argument.");
+
+  c = add_config_param(cmd->argv[0], 1, bool);
+  c->flags |= CF_MERGEDOWN;
+
+  return HANDLED(cmd);
+}
+
+MODRET set_rootlogin(cmd_rec *cmd) {
+  int bool = -1;
+  config_rec *c = NULL;
+
+  CHECK_ARGS(cmd,1);
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL|CONF_ANON);
+
+  if ((bool = get_boolean(cmd, 1)) == -1)
+    CONF_ERROR(cmd, "expected boolean argument.");
+
+  c = add_config_param(cmd->argv[0], 1, NULL);
+  c->argv[0] = pcalloc(c->pool, sizeof(unsigned char));
+  *((unsigned char *) c->argv[0]) = (unsigned char) bool;
+
+  c->flags |= CF_MERGEDOWN;
+  return HANDLED(cmd);
+}
+
+MODRET set_timeoutsession(cmd_rec *cmd) {
+  unsigned int seconds = 0, precedence = 0;
+  config_rec *c = NULL;
+
+  int ctxt = (cmd->config && cmd->config->config_type != CONF_PARAM ?
+     cmd->config->config_type : cmd->server->config_type ?
+     cmd->server->config_type : CONF_ROOT);
+
+  /* this directive must have either 1 or 3 arguments */
+  if (cmd->argc-1 != 1 && cmd->argc-1 != 3)
+    CONF_ERROR(cmd, "missing arguments");
+
+  CHECK_CONF(cmd, CONF_ROOT|CONF_ANON|CONF_VIRTUAL|CONF_GLOBAL);
+
+  /* Set the precedence for this config_rec based on its configuration
+   * context.
+   */
+  if (ctxt & CONF_GLOBAL)
+    precedence = 1;
+
+  /* these will never appear simultaneously */
+  else if (ctxt & CONF_ROOT || ctxt & CONF_VIRTUAL)
+    precedence = 2;
+
+  else if (ctxt & CONF_ANON)
+    precedence = 3;
+
+  if ((seconds = atoi(cmd->argv[1])) < 0) {
+    CONF_ERROR(cmd, pstrcat(cmd->tmp_pool,
+      "seconds must be greater than or equal to 0", NULL));
+
+  } else if (seconds == 0) {
+
+    /* do nothing */
+    return HANDLED(cmd);
+  }
+
+  if (cmd->argc-1 == 3) {
+    if (!strcmp(cmd->argv[2], "user") ||
+        !strcmp(cmd->argv[2], "group") ||
+        !strcmp(cmd->argv[2], "class")) {
+
+       /* no op */
+
+     } else
+       CONF_ERROR(cmd, pstrcat(cmd->tmp_pool, cmd->argv[0],
+         ": unknown classifier used: '", cmd->argv[2], "'", NULL));
+  }
+
+  if (cmd->argc-1 == 1) {
+    c = add_config_param(cmd->argv[0], 2, NULL);
+    c->argv[0] = pcalloc(c->pool, sizeof(unsigned int));
+    *((unsigned int *) c->argv[0]) = seconds;
+    c->argv[1] = pcalloc(c->pool, sizeof(unsigned int));
+    *((unsigned int *) c->argv[1]) = precedence;
+ 
+  } else if (cmd->argc-1 == 3) {
+    array_header *acl = NULL;
+    int argc = cmd->argc - 3;
+    char **argv = cmd->argv + 2;
+
+    acl = parse_group_expression(cmd->tmp_pool, &argc, argv);
+
+    c = add_config_param(cmd->argv[0], 0);
+    c->argc = argc + 2;
+
+    /* add 3 to argc for the argv of the config_rec: one for the
+     * seconds value, one for the precedence, one for the classifier,
+     * and one for the terminating NULL
+     */
+    c->argv = pcalloc(c->pool, ((argc + 4) * sizeof(char *)));
+
+    /* capture the config_rec's argv pointer for doing the by-hand
+     * population
+     */
+    argv = (char **) c->argv;
+
+    /* Copy in the seconds. */
+    *argv = pcalloc(c->pool, sizeof(unsigned int));
+    *((unsigned int *) *argv++) = seconds;
+
+    /* Copy in the precedence. */
+    *argv = pcalloc(c->pool, sizeof(unsigned int));
+    *((unsigned int *) *argv++) = precedence;
+
+    /* Copy in the classifier. */
+    *argv++ = pstrdup(c->pool, cmd->argv[2]);
+
+    /* now, copy in the expression arguments */
+    if (argc && acl) {
+      while (argc--) {
+        *argv++ = pstrdup(c->pool, *((char **) acl->elts));
+        acl->elts = ((char **) acl->elts) + 1;
+      }
+    }
+
+    /* don't forget the terminating NULL */
+    *argv = NULL;
+  }
+
+  c->flags |= CF_MERGEDOWN_MULTI;
+  return HANDLED(cmd);
+}
+
+MODRET set_useftpusers(cmd_rec *cmd) {
+  int bool = -1;
+  config_rec *c = NULL;
+
+  CHECK_ARGS(cmd, 1);
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL|CONF_ANON);
+
+  if ((bool = get_boolean(cmd, 1)) == -1)
+    CONF_ERROR(cmd, "expected boolean argument.");
+
+  c = add_config_param(cmd->argv[0], 1, (void *) bool);
+
+  c->flags |= CF_MERGEDOWN;
   return HANDLED(cmd);
 }
 
@@ -1825,27 +2190,54 @@ MODRET add_userdirroot (cmd_rec *cmd) {
   if ((bool = get_boolean(cmd, 1)) == -1)
     CONF_ERROR(cmd, "expected boolean argument.");
 
-  add_config_param("UserDirRoot", 1, (void *) bool);
+  add_config_param(cmd->argv[0], 1, (void *) bool);
   return HANDLED(cmd);
 }
 
+MODRET set_userpassword(cmd_rec *cmd) {
+  config_rec *c = NULL;
+
+  CHECK_ARGS(cmd, 2);
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL|CONF_ANON);
+
+  c = add_config_param_str(cmd->argv[0], 2, cmd->argv[1], cmd->argv[2]);
+  c->flags |= CF_MERGEDOWN;
+
+  return HANDLED(cmd);
+}
+
+/* Module API tables
+ */
+
 static conftable auth_conftab[] = {
+  { "AccessDenyMsg",		set_accessdenymsg,		NULL },
+  { "AccessGrantMsg",		set_accessgrantmsg,		NULL },
+  { "AnonRequirePassword",	set_anonrequirepassword,	NULL },
+  { "AnonymousGroup",		add_anonymousgroup,		NULL },
   { "AuthAliasOnly",		set_authaliasonly,		NULL },
-  { "DefaultRoot",		add_defaultroot,		NULL },
+  { "AuthUsingAlias",		set_authusingalias,		NULL },
   { "DefaultChdir",		add_defaultchdir,		NULL },
+  { "DefaultRoot",		add_defaultroot,		NULL },
+  { "GroupPassword",		set_grouppassword,		NULL },
   { "LoginPasswordPrompt",	set_loginpasswordprompt,	NULL },
+  { "MaxLoginAttempts",		set_maxloginattempts,		NULL },
+  { "RequireValidShell",	set_requirevalidshell,		NULL },
   { "RootLogin",		set_rootlogin,			NULL },
+  { "TimeoutSession",		set_timeoutsession,		NULL },
+  { "UseFtpUsers",		set_useftpusers,		NULL },
   { "UserDirRoot",		add_userdirroot,		NULL },
+  { "UserPassword",		set_userpassword,		NULL },
   { NULL,			NULL,				NULL }
 };
 
 static cmdtable auth_cmdtab[] = {
-  { PRE_CMD, C_USER, G_NONE, pre_cmd_user, FALSE, FALSE, CL_AUTH },
-  { CMD, C_USER, G_NONE, cmd_user,	FALSE,	FALSE, CL_AUTH },
-  { PRE_CMD, C_PASS, G_NONE, pre_cmd_pass, FALSE, FALSE, CL_AUTH },
-  { CMD, C_PASS, G_NONE, cmd_pass,	FALSE,  FALSE, CL_AUTH },
-  { CMD, C_ACCT, G_NONE, cmd_acct,	FALSE,  FALSE, CL_AUTH },
-  { CMD, C_REIN, G_NONE, cmd_rein,	FALSE,  FALSE, CL_AUTH },
+  { PRE_CMD,	C_USER,	G_NONE,	pre_cmd_user,	FALSE,	FALSE,	CL_AUTH },
+  { CMD,	C_USER,	G_NONE,	cmd_user,	FALSE,	FALSE,	CL_AUTH },
+  { PRE_CMD,	C_PASS,	G_NONE,	pre_cmd_pass,	FALSE,	FALSE,	CL_AUTH },
+  { CMD,	C_PASS,	G_NONE,	cmd_pass,	FALSE,	FALSE,	CL_AUTH },
+  { POST_CMD,	C_PASS,	G_NONE,	post_cmd_pass,	FALSE,	FALSE,	CL_AUTH },
+  { CMD,	C_ACCT,	G_NONE,	cmd_acct,	FALSE,	FALSE,	CL_AUTH },
+  { CMD,	C_REIN,	G_NONE,	cmd_rein,	FALSE,	FALSE,	CL_AUTH },
   { 0, NULL }
 };
 
