@@ -1,7 +1,8 @@
 /*
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
- * Copyright (C) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 2001, 2002 The ProFTPD Project team
  *  
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +26,7 @@
 
 /*
  * Data transfer module for ProFTPD
- * $Id: mod_xfer.c,v 1.65 2002/05/21 20:47:19 castaglia Exp $
+ * $Id: mod_xfer.c,v 1.77 2002/08/14 16:17:35 castaglia Exp $
  */
 
 /* History Log:
@@ -94,10 +95,10 @@ static void _log_transfer(char direction, char abort_flag) {
              direction,'r',session.user, abort_flag);
   }
 
-  log_debug(DEBUG1, "Transfer %s %d bytes in %d.%02d seconds.",
+  log_debug(DEBUG1, "Transfer %s %" PR_LU " bytes in %d.%02d seconds.",
 	    abort_flag == 'c' ? "completed:" : "aborted after",
-	    session.xfer.total_bytes,end_time.tv_sec,
-	    (end_time.tv_usec / 10000));
+	    session.xfer.total_bytes, (int) end_time.tv_sec,
+	    (int) (end_time.tv_usec / 10000));
 }
 
 /* This routine counts the difference in usec between timeval's
@@ -131,7 +132,7 @@ static float _rate_diffusec(struct timeval tlast, struct timeval t) {
  *              rate_bps:       max byte / sec bandwidth allowed
  *              rate_hardbps:   if FALSE then forces BPS only after FreeBytes
  */
-static void _rate_throttle(unsigned long rate_pos, long rate_bytes,
+static void _rate_throttle(off_t rate_pos, off_t rate_bytes,
 			   struct timeval rate_tvlast,
 			   long rate_freebytes, long rate_bps,
 			   int rate_hardbps)
@@ -142,16 +143,14 @@ static void _rate_throttle(unsigned long rate_pos, long rate_bytes,
   float dtime, wtime;
 
   /* no rate control unless more than free bytes DL'ed */
-  log_debug(DEBUG5,
-	    "_rate_throttle: rate_bytes=%ld  rate_pos=%ld  rate_freebytes=%ld "
-	    "rate_bps=%ld  rate_hardbps=%i.",
-	    rate_bytes, rate_pos,
-	    rate_freebytes, rate_bps, rate_hardbps);
+  log_debug(DEBUG5, "_rate_throttle: rate_bytes=%" PR_LU
+    " rate_pos=%" PR_LU " rate_freebytes=%ld rate_bps=%ld rate_hardbps=%d.",
+    rate_bytes, rate_pos, rate_freebytes, rate_bps, rate_hardbps);
 
   if(rate_pos < rate_freebytes)
     return;
   
-  while(1) {
+  while (1) {
     gettimeofday(&rate_tv, NULL);
   
     if(!rate_hardbps)
@@ -195,11 +194,14 @@ static void _rate_throttle(unsigned long rate_pos, long rate_bytes,
        * jss 2/20/01
        */
     
-      if(select(0, NULL, NULL, NULL, &rate_tv) < 0) {
+      if (select(0, NULL, NULL, NULL, &rate_tv) < 0) {
         if(errno != EINTR) {
           log_pri(LOG_WARNING, "Unable to throttle bandwidth: %s.",
 	          strerror(errno));
-        } else continue;
+        } else {
+          pr_handle_signals();
+          continue;
+        }
       }
     }
     break;
@@ -216,7 +218,7 @@ static int _transmit_normal(char *buf, long bufsize) {
 }
 
 #ifdef HAVE_SENDFILE
-static int _transmit_sendfile(int rate_bps, unsigned long count, off_t *offset,
+static int _transmit_sendfile(int rate_bps, off_t count, off_t *offset,
 			       pr_sendfile_t *retval) {
   
   /* We don't use sendfile() if:
@@ -272,7 +274,7 @@ static int _transmit_sendfile(int rate_bps, unsigned long count, off_t *offset,
 }
 #endif /* HAVE_SENDFILE */
 
-static long _transmit_data(int rate_bps, unsigned long count, off_t offset,
+static long _transmit_data(int rate_bps, off_t count, off_t offset,
 			   char *buf, long bufsize) {
 #ifdef HAVE_SENDFILE
   pr_sendfile_t retval;
@@ -404,16 +406,6 @@ MODRET pre_cmd_stor(cmd_rec *cmd) {
     return ERROR(cmd);
   }
   
-  /* No PORT command has been issued.
-   */
-  if(!(session.flags & SF_PASSIVE) &&
-     (session.d != NULL || !(session.flags & SF_PORT))) {
-    add_response_err(R_503, "No PORT command issued first.");
-    return ERROR(cmd);
-  }
-
-  session.flags &= ~SF_PORT;
-  
   dir = dir_best_path(cmd->tmp_pool,cmd->arg);
 
   if(!dir || !dir_check(cmd->tmp_pool,cmd->argv[0],cmd->group,dir,NULL)) {
@@ -530,7 +522,123 @@ MODRET pre_cmd_stor(cmd_rec *cmd) {
     session.xfer.xfer_type = STOR_HIDDEN;
   }
 
+  return HANDLED(cmd);
+}
 
+/* pre_cmd_stou is a PRE_CMD handler which changes the uploaded filename
+ * to a unique one, after making the requisite security and authorization
+ * checks.
+ */
+MODRET pre_cmd_stou(cmd_rec *cmd) {
+  config_rec *c = NULL;
+  privdata_t *priv = NULL;
+  char *prefix = "ftp", *filename = NULL;
+  int tmpfd;
+  mode_t mode;
+
+  /* Some FTP clients are "broken" in that they will send a filename
+   * along with STOU.  Technically this violates RFC959, but for now, just
+   * ignore that filename.  Stupid client implementors.
+   */
+
+  if (cmd->argc > 2) {
+    add_response_err(R_500, "'%s' not understood.", get_full_cmd(cmd));
+    return ERROR(cmd);
+  }
+
+  /* Watch for STOU preceded by REST, which makes no sense.
+   *
+   *   REST: session.restart_pos > 0
+   */
+  if (session.restart_pos) {
+    add_response_err(R_550, "STOU incompatible with REST");
+    return ERROR(cmd);
+  }
+
+  /* Generate the filename to be stored, depending on the configured
+   * unique filename prefix.
+   */
+  if ((c = find_config(CURRENT_CONF, CONF_PARAM, "StoreUniquePrefix",
+      FALSE)) != NULL)
+    prefix = c->argv[0];
+
+  /* Now, construct the unique filename using the cmd_rec's pool, the
+   * prefix, and mkstemp().
+   */
+  filename = pstrcat(cmd->pool, prefix, "XXXXXX", NULL);
+
+  if ((tmpfd = mkstemp(filename)) < 0) {
+    log_pri(LOG_ERR, "error: unable to use mkstemp(): %s", strerror(errno));
+
+    /* If we can't guarantee a unique filename, refuse the command. */
+    add_response_err(R_450, "%s: unable to generate unique filename",
+      cmd->argv[0]); 
+    return ERROR(cmd);
+
+  } else {
+    cmd->arg = filename;
+
+    /* Close the unique file.  This introduces a small race condition
+     * between the time this function returns, and the STOU CMD handler
+     * opens the unique file, but this may have to do, as closing that
+     * race would involve some major restructuring.
+     */
+    close(tmpfd);
+  }
+
+  /* It's OK to reuse the char * pointer for filename.
+   */
+  filename = dir_best_path(cmd->tmp_pool, cmd->arg);
+
+  if (!filename || !dir_check(cmd->tmp_pool, cmd->argv[0], cmd->group,
+      filename, NULL)) {
+    add_response_err(R_550, "%s: %s", cmd->arg, strerror(errno));
+    return ERROR(cmd);
+  }
+
+  mode = file_mode(filename);
+
+  if (mode &&
+      session.xfer.xfer_type != STOR_APPEND &&
+      get_param_int(CURRENT_CONF,"AllowOverwrite",FALSE) != 1) {
+    add_response_err(R_550, "%s: Overwrite permission denied", cmd->arg);
+    return ERROR(cmd);
+  }
+
+  /* Not likely to _not_ be a regular file, but just to be certain...
+   */
+  if (mode && !S_ISREG(mode)) {
+    add_response_err(R_550, "%s: Not a regular file", cmd->arg);
+    return ERROR(cmd);
+  }
+
+  /* Otherwise everthing is good */
+  priv = mod_privdata_alloc(cmd, "stor_filename", strlen(filename) + 1);
+  sstrncpy(priv->value.str_val, filename, strlen(filename) + 1);
+
+  session.xfer.xfer_type = STOR_UNIQUE;
+ 
+  return HANDLED(cmd);
+}
+
+/* post_cmd_stou is a POST_CMD handler that changes the mode of the
+ * STOU file from 0600, which is what mkstemp() makes it, to 0666,
+ * the default for files uploaded via STOR.  This is to prevent users
+ * from being surprised.
+ */
+MODRET post_cmd_stou(cmd_rec *cmd) {
+
+  /* This is the same mode as used in src/fs.c.  Should probably be
+   * available as a macro.
+   */
+  mode_t mode = 0666;
+
+  if (fs_chmod(cmd->arg, mode) < 0) {
+
+    /* Not much to do but log the error. */
+    log_pri(LOG_ERR, "error: unable to chmod '%s': %s", cmd->arg,
+      strerror(errno));
+  }
 
   return HANDLED(cmd);
 }
@@ -553,7 +661,7 @@ MODRET cmd_stor(cmd_rec *cmd)
   char *lbuf;
   int bufsize,len;
   struct stat sbuf;
-  unsigned long respos = 0;
+  off_t respos = 0;
   privdata_t *p, *p_hidden;
 
 #if defined(HAVE_REGEX_H) && defined(HAVE_REGCOMP)
@@ -678,9 +786,7 @@ MODRET cmd_stor(cmd_rec *cmd)
       
     session.xfer.file_size = respos;
 
-    /* first, make sure the uploaded file has the requested ownership
-     * -tj, 2001-10-18
-     */
+    /* First, make sure the uploaded file has the requested ownership. */
     _stor_chown();
 
     if(data_open(cmd->arg,NULL,IO_READ,0) < 0) {
@@ -694,7 +800,7 @@ MODRET cmd_stor(cmd_rec *cmd)
     lbuf = (char*) palloc(cmd->tmp_pool, bufsize);
     
     gettimeofday(&rate_tvstart, NULL);
-    while((len = data_xfer(lbuf, bufsize)) > 0) {
+    while ((len = data_xfer(lbuf, bufsize)) > 0) {
       if(XFER_ABORTED)
         break;
 
@@ -714,34 +820,38 @@ MODRET cmd_stor(cmd_rec *cmd)
       }
     }
 
-    if(XFER_ABORTED) {
+    if (XFER_ABORTED) {
       _stor_abort();
-      data_abort(0,0);
+      data_abort(0, 0);
       return ERROR(cmd);
-    } else if(len < 0) {
+
+    } else if (len < 0) {
       _stor_abort();
-      data_abort(session.d->inf->xerrno,FALSE);
+      data_abort(session.d->inf->xerrno, FALSE);
       return ERROR(cmd);
+
     } else {
-      if(session.xfer.path && session.xfer.path_hidden) {
-        if(fs_rename(session.xfer.path_hidden,session.xfer.path) != 0) {
+      _stor_done();
+
+      if (session.xfer.path && session.xfer.path_hidden) {
+
+        if (fs_rename(session.xfer.path_hidden, session.xfer.path) != 0) {
           /* This should only fail on a race condition with a chmod/chown
            * or if STOR_APPEND is on and the permissions are squirrely.
            * The poor user will have to re-upload, but we've got more important
            * problems to worry about and this failure should be fairly rare.
            */
-           log_pri(LOG_WARNING, "Rename of %s to %s failed: %s.",
-             session.xfer.path_hidden, session.xfer.path, strerror(errno));
+          log_pri(LOG_WARNING, "Rename of %s to %s failed: %s.",
+            session.xfer.path_hidden, session.xfer.path, strerror(errno));
 
-           add_response_err(R_550,"%s: rename of hidden file %s failed: %s",
-             session.xfer.path, session.xfer.path_hidden, strerror(errno));
+          add_response_err(R_550,"%s: rename of hidden file %s failed: %s",
+            session.xfer.path, session.xfer.path_hidden, strerror(errno));
 
-           fs_unlink(session.xfer.path_hidden);
+          fs_unlink(session.xfer.path_hidden);
 
-           return ERROR(cmd);
-         }
+          return ERROR(cmd);
+        }
       }
-      _stor_done();
       data_close(FALSE);
     }
   }
@@ -837,8 +947,8 @@ MODRET cmd_retr(cmd_rec *cmd)
   privdata_t *p;
   long bufsize, len = 0;
   long rate_hardbps = 0;
-  unsigned long respos = 0,cnt = 0,cnt_steps = 0,cnt_next = 0;
-  long rate_bytes = 0, rate_freebytes = 0, rate_bps = 0;
+  off_t respos = 0, cnt = 0, cnt_steps = 0, cnt_next = 0, rate_bytes = 0;
+  long rate_freebytes = 0, rate_bps = 0;
   
   if((rate_bps = get_param_int(CURRENT_CONF, "RateReadBPS", FALSE)) == -1)
     rate_bps = 0;
@@ -888,7 +998,7 @@ MODRET cmd_retr(cmd_rec *cmd)
     data_init(cmd->arg,IO_WRITE);
     
     session.xfer.path = pstrdup(session.xfer.p,dir);
-    session.xfer.file_size = (unsigned long) sbuf.st_size;
+    session.xfer.file_size = sbuf.st_size;
     cnt_steps = session.xfer.file_size / 100;
     if(cnt_steps == 0)
       cnt_steps = 1;
@@ -910,12 +1020,12 @@ MODRET cmd_retr(cmd_rec *cmd)
 
     gettimeofday(&rate_tvstart, NULL);
     
-    while(cnt != session.xfer.file_size) {
-      if(XFER_ABORTED)
+    while (cnt != session.xfer.file_size) {
+      if (XFER_ABORTED)
         break;
       
       /* INSERT CODE HERE */
-      if((len = _transmit_data(rate_bps, cnt, respos, lbuf, bufsize)) == 0)
+      if ((len = _transmit_data(rate_bps, cnt, respos, lbuf, bufsize)) == 0)
 	break;
       
       if(len < 0) {
@@ -1096,13 +1206,6 @@ cmd_smnt(cmd_rec *cmd)
 }
 
 
-MODRET
-cmd_stou(cmd_rec *cmd)
-{
-	add_response(R_502, "STOU command not implemented.");
-	return HANDLED(cmd);
-}
-
 MODRET xfer_err_cleanup(cmd_rec *cmd) {
   if (session.xfer.p)
     destroy_pool(session.xfer.p);
@@ -1159,8 +1262,43 @@ int xfer_init_child(void)
   return 0;
 }
 
-MODRET add_ratenum(cmd_rec *cmd)
-{
+/* Configuration handlers
+ */
+
+MODRET set_deleteabortedstores(cmd_rec *cmd) {
+  int bool = -1;
+  config_rec *c = NULL;
+
+  CHECK_ARGS(cmd, 1);
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL|CONF_ANON|
+    CONF_DIR|CONF_DYNDIR);
+
+  if ((bool = get_boolean(cmd, 1)) == -1)
+    CONF_ERROR(cmd, "expected boolean parameter");
+
+  c = add_config_param(cmd->argv[0], 1, (void *) bool);
+  c->flags |= CF_MERGEDOWN;
+
+  return HANDLED(cmd);
+}
+
+MODRET set_hiddenstores(cmd_rec *cmd) {
+  int bool = -1;
+  config_rec *c = NULL;
+
+  CHECK_ARGS(cmd, 1);
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL|CONF_ANON|CONF_DIR);
+
+  if ((bool = get_boolean(cmd, 1)) == -1)
+    CONF_ERROR(cmd, "expected Boolean parameter");
+
+  c = add_config_param(cmd->argv[0], 1, (void *) bool);
+  c->flags |= CF_MERGEDOWN;
+
+  return HANDLED(cmd);
+}
+
+MODRET add_ratenum(cmd_rec *cmd) {
   config_rec *c;
   long ratenum;
   char *endp;
@@ -1188,8 +1326,7 @@ MODRET add_ratenum(cmd_rec *cmd)
   return HANDLED(cmd);
 }
 
-MODRET add_ratebool(cmd_rec *cmd)
-{
+MODRET add_ratebool(cmd_rec *cmd) {
   config_rec *c;
   int b;
 
@@ -1207,17 +1344,39 @@ MODRET add_ratebool(cmd_rec *cmd)
   return HANDLED(cmd);
 }
 
-conftable xfer_config[] = {
+MODRET set_storeuniqueprefix(cmd_rec *cmd) {
+  config_rec *c = NULL;
+
+  CHECK_ARGS(cmd, 1);
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL|CONF_ANON|
+    CONF_DIR|CONF_DYNDIR);
+
+  /* make sure there are no slashes in the prefix */
+  if (strchr(cmd->argv[1], '/') != NULL)
+    CONF_ERROR(cmd, pstrcat(cmd->tmp_pool, "no slashes allowed in prefix: '",
+      cmd->argv[1], "'", NULL));
+
+  c = add_config_param_str(cmd->argv[0], 1, (void *) cmd->argv[1]);
+  c->flags |= CF_MERGEDOWN;
+
+  return HANDLED(cmd);
+}
+
+static conftable xfer_conftab[] = {
+  { "DeleteAbortedStores",	set_deleteabortedstores,	},
+  { "HiddenStor",		set_hiddenstores,		},
+  { "HiddenStores",		set_hiddenstores,		},
   { "RateReadBPS",		add_ratenum,                 },
   { "RateReadFreeBytes",	add_ratenum,	             },
   { "RateReadHardBPS",		add_ratebool,                },
   { "RateWriteBPS",		add_ratenum,                 },
   { "RateWriteFreeBytes",	add_ratenum,	             },
   { "RateWriteHardBPS",		add_ratebool,                },
+  { "StoreUniquePrefix",	set_storeuniqueprefix,		},
   { NULL }
 };
 
-static cmdtable xfer_commands[] = {
+static cmdtable xfer_cmdtab[] = {
   { CMD,     C_TYPE,	G_NONE,	 cmd_type,	TRUE,	FALSE, CL_MISC },
   { CMD,     C_STRU,	G_NONE,	 cmd_stru,	TRUE,	FALSE, CL_MISC },
   { CMD,     C_MODE,	G_NONE,	 cmd_mode,	TRUE,	FALSE, CL_MISC },
@@ -1226,17 +1385,20 @@ static cmdtable xfer_commands[] = {
   { PRE_CMD, C_RETR,	G_READ,	 pre_cmd_retr,	TRUE,	FALSE },
   { CMD,     C_RETR,	G_READ,	 cmd_retr,	TRUE,	FALSE, CL_READ },
   { LOG_CMD, C_RETR,	G_NONE,	 log_retr,	FALSE,  FALSE },
+  { LOG_CMD_ERR, C_RETR,G_NONE,  xfer_err_cleanup,  FALSE,  FALSE },
   { PRE_CMD, C_STOR,	G_WRITE, pre_cmd_stor,	TRUE,	FALSE },
   { CMD,     C_STOR,	G_WRITE, cmd_stor,	TRUE,	FALSE, CL_WRITE },
   { LOG_CMD, C_STOR,    G_NONE,	 log_stor,	FALSE,  FALSE },
   { LOG_CMD_ERR, C_STOR,G_NONE,  xfer_err_cleanup,  FALSE,  FALSE },
-  { LOG_CMD_ERR, C_RETR,G_NONE,  xfer_err_cleanup,  FALSE,  FALSE },
-  { CMD,     C_STOU,	G_WRITE, cmd_stou,	TRUE,	FALSE, CL_WRITE },
-  { PRE_CMD, C_APPE,	G_WRITE, pre_cmd_appe,	TRUE,	FALSE },
+  { PRE_CMD, C_STOU,	G_WRITE, pre_cmd_stou,	TRUE,	FALSE },
+  { CMD,     C_STOU,	G_WRITE, cmd_stor,	TRUE,	FALSE, CL_WRITE },
+  { POST_CMD,C_STOU,	G_WRITE, post_cmd_stou,	FALSE,	FALSE },
+  { LOG_CMD, C_STOU,	G_NONE,  log_stor,	FALSE,	FALSE },
   { LOG_CMD_ERR, C_STOU,G_NONE,  xfer_err_cleanup,  FALSE,  FALSE },
-  { LOG_CMD_ERR, C_APPE,G_NONE,  xfer_err_cleanup,  FALSE,  FALSE },
+  { PRE_CMD, C_APPE,	G_WRITE, pre_cmd_appe,	TRUE,	FALSE },
   { CMD,     C_APPE,	G_WRITE, cmd_stor,	TRUE,	FALSE, CL_WRITE },
   { LOG_CMD, C_APPE,	G_NONE,  log_stor,	FALSE,  FALSE },
+  { LOG_CMD_ERR, C_APPE,G_NONE,  xfer_err_cleanup,  FALSE,  FALSE },
   { CMD,     C_ABOR,	G_NONE,	 cmd_abor,	TRUE,	TRUE,  CL_MISC  },
   { CMD,     C_REST,	G_NONE,	 cmd_rest,	TRUE,	FALSE, CL_MISC  },
   { 0,NULL }
@@ -1246,8 +1408,8 @@ module xfer_module = {
   NULL,NULL,				/* Always NULL */
   0x20,					/* API Version */
   "xfer",				/* Module name */
-  xfer_config,
-  xfer_commands,
+  xfer_conftab,
+  xfer_cmdtab,
   NULL,
   NULL,
   xfer_init_child
