@@ -1,7 +1,8 @@
 /*
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
- * Copyright (C) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 2001, 2002 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +25,7 @@
  */
 
 /* Authentication front-end for ProFTPD
- * $Id: auth.c,v 1.10 2001/12/13 20:35:50 flood Exp $
+ * $Id: auth.c,v 1.14 2002/06/25 16:37:36 castaglia Exp $
  */
 
 #include "conf.h"
@@ -61,6 +62,8 @@ static modret_t *_dispatch_auth(cmd_rec *cmd, char *match)
 
   m = mod_find_auth_symbol(match,&cmd->symtable_index,NULL);
   while(m) {
+    log_debug(DEBUG6, "dispatching auth request \"%s\" to module mod_%s",
+      match, m->m->name);
     mr = call_module_auth(m->m,m->handler,cmd);
     if(MODRET_ISHANDLED(mr) || MODRET_ISERROR(mr))
       break;
@@ -477,9 +480,8 @@ int auth_getgroups(pool *p, const char *name, array_header **group_ids,
   modret_t *mr = NULL;
   int res = -1;
   
-  /* allocate memory for the array_headers of GIDs and group names
-   */
-    if (group_ids)
+  /* Allocate memory for the array_headers of GIDs and group names. */
+  if (group_ids)
     *group_ids = make_array(permanent_pool, 2, sizeof(gid_t));
   
   if (group_names)
@@ -493,13 +495,12 @@ int auth_getgroups(pool *p, const char *name, array_header **group_ids,
   if (MODRET_ISHANDLED(mr) && MODRET_HASDATA(mr)) {
     res = (int) mr->data;
 
-    /* NOTE: the number of groups returned should, barring error,
+    /* Note: the number of groups returned should, barring error,
      * always be at least 1, as per getgroups(2) behavior.  This one
      * ID is present because it is the primary group membership set in
      * struct passwd, from /etc/passwd.  This will need to be documented
      * for the benefit of auth_getgroup() implementors.
-     * -tj, 2001-10-12
-         */
+     */
   }
  
   if (cmd->tmp_pool) {
@@ -511,6 +512,9 @@ int auth_getgroups(pool *p, const char *name, array_header **group_ids,
 }
 
 int set_groups(pool *p, gid_t primary_gid, array_header *suppl_gids) {
+#ifndef HAVE_SETGROUPS
+  return 0;
+#else /* HAVE_SETGROUPS */
   gid_t *process_gids = NULL, *group_ids = NULL;
   size_t ngids = 0;
   int i = 0, res = 0;
@@ -543,7 +547,8 @@ int set_groups(pool *p, gid_t primary_gid, array_header *suppl_gids) {
    */
   if ((res = setgid(primary_gid)) < 0)
     return res;
- 
+
   return res;
+#endif /* HAVE_SETGROUPS */
 }
 
