@@ -1,7 +1,8 @@
 /*
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
- * Copyright (C) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 2001, 2002 The ProFTPD Project team
  *  
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +26,7 @@
  
 /*
  * Data connection management functions
- * $Id: data.c,v 1.30 2002/05/21 20:47:22 castaglia Exp $
+ * $Id: data.c,v 1.39 2002/08/14 16:17:35 castaglia Exp $
  */
 
 #include "conf.h"
@@ -66,7 +67,7 @@ static int stalled_timeout(CALLBACK_FRAME) {
 /* this signal is raised if we get OOB data on the control connection, and
  * a data transfer is in progress
  */
-RETSIGTYPE data_urgent(int sig) {
+static RETSIGTYPE data_urgent(int sig) {
   if(curf) {
     session.flags |= SF_ABORT;
     io_abort(curf);
@@ -141,7 +142,7 @@ static void _data_new_xfer(char *filename, int direction) {
   session.xfer.buflen = 0;
 }
 
-static int _data_pasv_open(char *reason, unsigned long size) {
+static int _data_pasv_open(char *reason, off_t size) {
   conn_t *c;
   int rev;
   
@@ -167,22 +168,43 @@ static int _data_pasv_open(char *reason, unsigned long size) {
     inet_setnonblock(session.pool,c);
     session.d = c;
     
-    if(size) {
-      send_response(R_150,
-		    "Opening %s mode data connection for %s (%lu bytes).",
-		    MODE_STRING, reason, size);
+    if (session.xfer.xfer_type != STOR_UNIQUE) { 
+      if (size)
+        send_response(R_150, "Opening %s mode data connection for %s "
+          "(%" PR_LU " bytes)", MODE_STRING, reason, size);
+      else
+        send_response(R_150,"Opening %s mode data connection for %s",
+          MODE_STRING, reason);
+
     } else {
-      send_response(R_150,"Opening %s mode data connection for %s",
-		    MODE_STRING, reason);
+
+      /* Format of 150 responses for STOU is explicitly dictated by
+       * RFC 1123:
+       *
+       *  4.1.2.9  STOU Command: RFC-959 Section 4.1.3
+       *
+       *    The STOU command stores into a uniquely named file.  When it
+       *    receives an STOU command, a Server-FTP MUST return the
+       *    actual file name in the "125 Transfer Starting" or the "150
+       *    Opening Data Connection" message that precedes the transfer
+       *    (the 250 reply code mentioned in RFC-959 is incorrect).  The
+       *    exact format of these messages is hereby defined to be as
+       *    follows:
+       *
+       *        125 FILE: pppp
+       *        150 FILE: pppp
+       *
+       *    where pppp represents the unique pathname of the file that
+       *    will be written.
+       */
+      send_response(R_150, "FILE: %s", reason);
     }
     
     return 0;
   }
   
-  /* Check for error conditions.
-   * - MacGyver
-   */
-  if(c && c->mode == CM_ERROR) {
+  /* Check for error conditions. */
+  if (c && c->mode == CM_ERROR) {
     log_pri(LOG_ERR,
 	    "Error: unable to accept an incoming data connection (%s)",
 	    strerror(c->xerrno));
@@ -195,7 +217,7 @@ static int _data_pasv_open(char *reason, unsigned long size) {
   return -1;
 }
 
-static int _data_active_open(char *reason, unsigned long size) {
+static int _data_active_open(char *reason, off_t size) {
   conn_t *c;
   int rev;
   
@@ -235,14 +257,36 @@ static int _data_active_open(char *reason, unsigned long size) {
 	      inet_ntoa(*session.d->remote_ipaddr),
 	      session.d->remote_port);
     
-    if(size) {
-      send_response(R_150,
-		    "Opening %s mode data connection for %s (%lu bytes).",
-		    MODE_STRING,reason,size);
+    if (session.xfer.xfer_type != STOR_UNIQUE) {
+      if (size)
+        send_response(R_150, "Opening %s mode data connection for %s "
+          "(%" PR_LU " bytes)", MODE_STRING, reason, size);
+      else
+        send_response(R_150, "Opening %s mode data connection for %s",
+          MODE_STRING, reason);
+
     } else {
-      send_response(R_150,
-		    "Opening %s mode data connection for %s.",
-		    MODE_STRING,reason);
+
+      /* Format of 150 responses for STOU is explicitly dictated by
+       * RFC 1123:
+       *
+       *  4.1.2.9  STOU Command: RFC-959 Section 4.1.3
+       *
+       *    The STOU command stores into a uniquely named file.  When it
+       *    receives an STOU command, a Server-FTP MUST return the
+       *    actual file name in the "125 Transfer Starting" or the "150
+       *    Opening Data Connection" message that precedes the transfer
+       *    (the 250 reply code mentioned in RFC-959 is incorrect).  The
+       *    exact format of these messages is hereby defined to be as
+       *    follows:
+       *
+       *        125 FILE: pppp
+       *        150 FILE: pppp
+       *
+       *    where pppp represents the unique pathname of the file that
+       *    will be written.
+       */
+      send_response(R_150, "FILE: %s", reason);
     }
     
     inet_close(session.pool,session.d);
@@ -278,8 +322,7 @@ void data_init(char *filename, int direction) {
   }
 }
 
-int data_open(char *filename, char *reason, int direction,
-	      unsigned long size) {
+int data_open(char *filename, char *reason, int direction, off_t size) {
   struct sigaction	act;
   int			ret = 0;
   
@@ -362,8 +405,9 @@ int data_open(char *filename, char *reason, int direction,
 /* close == successful transfer */
 void data_close(int quiet) {
   curf = NULL;
-  if(session.d) {
-    inet_close(session.pool,session.d);
+
+  if (session.d) {
+    inet_lingering_close(session.pool, session.d, TUNABLE_TIMEOUTLINGER);
     session.d = NULL;
   }
   
@@ -395,8 +439,8 @@ void data_close(int quiet) {
  */
 void data_cleanup(void) {
   /* sanity check */
-  if(session.d) {
-    inet_close(session.pool,session.d);
+  if (session.d) {
+    inet_lingering_close(session.pool, session.d, TUNABLE_TIMEOUTLINGER);
     session.d = NULL;
   }
 
@@ -404,8 +448,6 @@ void data_cleanup(void) {
     destroy_pool(session.xfer.p);
   
   memset(&session.xfer,0,sizeof(session.xfer));
-  
-  session.data_port = session.c->remote_port - 1;
 }
 
 /* In order to avoid clearing the transfer counters in session.xfer, we don't
@@ -418,8 +460,8 @@ void data_abort(int err, int quiet) {
   int true_abort = XFER_ABORTED;
   
   curf = NULL;
-  if(session.d) {
-    inet_close(session.pool,session.d);
+  if (session.d) {
+    inet_lingering_close(session.pool, session.d, TUNABLE_TIMEOUTLINGER);
     session.d = NULL;
   }
   
@@ -822,7 +864,9 @@ pr_sendfile_t data_sendfile(int retr_fd, off_t *offset, size_t count) {
        *
        * For obvious reasons, HP/UX sendfile is not supported yet - jss
        */
-      if(errno == EINTR) {
+      if (errno == EINTR) {
+        pr_handle_signals();
+
 	/* If we got everything in this transaction, we're done.
 	 */
 	if((count -= len) <= 0)
