@@ -1,7 +1,8 @@
 /*
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
- * Copyright (C) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 2001, 2002 The ProFTPD Project team
  *  
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,15 +24,9 @@
  * the source code for OpenSSL in the source distribution.
  */
 
-/* Read configuration file(s), and manage server/configuration
- * structures.
- * $Id: dirtree.c,v 1.53 2002/05/21 20:47:22 castaglia Exp $
- */
-
-/* History:
- * 5/1/97 0.99.0pl2
- *  Used to be named "config.c", renamed to dirtree.c (directive
- *  tree) so as not to conflict with GNU autoconf's top-level config.h.
+/* Read configuration file(s), and manage server/configuration structures.
+ *
+ * $Id: dirtree.c,v 1.64 2002/08/28 16:00:22 castaglia Exp $
  */
 
 #include "conf.h"
@@ -67,17 +62,28 @@ static void _mergedown(xaset_t*,int);
 /* Used by get_param_int_next & get_param_ptr_next as "placeholders" */
 static config_rec *_last_param_int = NULL;
 static config_rec *_last_param_ptr = NULL;
-static int _kludge_disable_umask = 0;
+static unsigned char _kludge_disable_umask = 0;
+
+array_header *server_defines = NULL;
 
 /* Used only while reading configuration files */
 
-struct {
+static struct {
   pool *tpool;
   array_header *sstack,*cstack;
   server_rec **curserver;
   config_rec **curconfig;
 } conf;
 
+typedef struct config_stream_struc {
+  struct config_stream_struc *cs_next;
+  pool *cs_pool;
+  FILE *cs_file;
+  unsigned int cs_lineno;
+} config_stream_t;
+
+static config_stream_t *config_stream_stack = NULL;
+
 /* Imported this function from modules/mod_ls.c -- it belongs more with the
  * dir_* functions here, rather than the ls_* functions there.
  */
@@ -152,14 +158,30 @@ int dir_check_hidden(const char *path) {
   return FALSE;	
 }
 
-void kludge_disable_umask(void)
-{
-  _kludge_disable_umask = 1;
+int define_exists(const char *definition) {
+  char **defines = NULL;
+  register unsigned int i = 0;
+
+  /* Check the list of specified definitions, if present.
+   */
+  if (server_defines) {
+    defines = (char **) server_defines->elts;
+    for (i = 0; i < server_defines->nelts; i++) {
+      if (defines[i] && !strcmp(defines[i], definition))
+        return TRUE;
+    }
+  }
+
+  /* default */
+  return FALSE;
 }
 
-void kludge_enable_umask(void)
-{
-  _kludge_disable_umask = 0;
+void kludge_disable_umask(void) {
+  _kludge_disable_umask = TRUE;
+}
+
+void kludge_enable_umask(void) {
+  _kludge_disable_umask = FALSE;
 }
 
 char *get_word(char **cp)
@@ -199,39 +221,149 @@ char *get_word(char **cp)
   return ret;
 }
 
-cmd_rec *get_config_cmd(pool *ppool, FILE *fp, int *line)
-{
-  char buf[1024] = {'\0'}, *cp, *wrd;
-  cmd_rec *newcmd;
-  pool *newpool;
-  array_header *tarr;
-  int i;
-  
-  while(fgets(buf, sizeof(buf) - 1, fp)) {
-    if(line != NULL)
-      (*line)++;
-    
-    i = strlen(buf);
-    if(i && buf[i - 1] == '\n')
-      buf[i - 1] = '\0';
-    
-    for(cp = buf; *cp && isspace((UCHAR)*cp); cp++) ;
+static config_stream_t *push_config_stream(FILE *filep, unsigned int lineno) {
+  pool *tmp_pool = make_sub_pool(permanent_pool);
+  config_stream_t *cs = pcalloc(tmp_pool, sizeof(config_stream_t));
 
-    if(*cp == '#' || !*cp)		/* Comment or blank line */
+  cs->cs_next = NULL;
+  cs->cs_pool = tmp_pool;
+  cs->cs_file = filep;
+  cs->cs_lineno = lineno;
+
+  if (!config_stream_stack)
+    config_stream_stack = cs;
+
+  else {
+    cs->cs_next = config_stream_stack;
+    config_stream_stack = cs;
+  }
+
+  return cs;
+}
+
+static void pop_config_stream(void) {
+  config_stream_t *cs = config_stream_stack;
+  config_stream_stack = cs->cs_next;
+
+  destroy_pool(cs->cs_pool);
+}
+
+/* get_line() is an fgets() with backslash-newline stripping, copied from
+ * Wietse Venema's tcpwrapppers-7.6 code.  The extra *lineno argument is
+ * needed, at the moment, to properly track which line of the configuration
+ * file is being read in, so that errors can be reported with line numbers
+ * correctly.
+ */
+char *get_line(char *buf, int buflen, FILE *filep, unsigned int *lineno) {
+  int inlen;
+  char *start = buf;
+
+  while (fgets(buf, buflen, filep)) {
+    inlen = strlen(buf);
+
+    if (inlen >= 1 && buf[inlen - 1] == '\n') {
+      (*lineno)++;
+
+      if (inlen >= 2 && buf[inlen - 2] == '\\') {
+        inlen -= 2;
+
+      } else
+        return start;
+    }
+
+    buf += inlen;
+    buflen -= inlen;
+    buf[0] = 0;
+  }
+
+  return (buf > start ? start : 0);
+}
+
+/* This functions returns the next line from the configuration stream,
+ * skipping commented-out lines and trimming trailing and leading whitespace,
+ * returning, in effect, the next line of configuration data on which to
+ * act.  At present, the configuration stream is indicated by the static
+ * config_stream FILE pointer -- in the future, this _will_ change to a
+ * more generic and flexible configuration stream data type (eg confstream_t).
+ * This function has the advantage that it can be called by functions that
+ * don't have access to that FILE pointer, such as the <IfDefine> and <IfModule>
+ * configuration handlers.  In the future, the requirement will be that
+ * functions wishing to access the configuration stream _must_ call
+ * set_config_stream() prior to calling all configuration stream functions
+ * (of which this is one of but several potential such functions).
+ */
+char *get_config_line(char *buf, size_t len) {
+
+  /* Always use the config stream at the top of the stack.
+   */
+  config_stream_t *cs = config_stream_stack;
+ 
+  if (!cs->cs_file)
+    return NULL;
+
+  /* Check for error conditions. */
+  while ((get_line(buf, len, cs->cs_file, &(cs->cs_lineno))) != NULL) {
+    char *bufp;
+    int buflen = strlen(buf);
+
+    /* trim off the trailing newline, if present
+     */
+    if (buflen && buf[buflen - 1] == '\n')
+      buf[buflen - 1] = '\0';
+    
+    /* trim off any leading whitespace
+     */
+    for (bufp = buf; *bufp && isspace(*bufp); bufp++);
+
+    /* check for commented or blank lines at this point, and just continue on
+     * to the next configuration line if found.  If not, return the 
+     * configuration line
+     */
+    if (*bufp == '#' || !*bufp)
       continue;
 
+    else {
+
+      /* copy the value of bufp back into the pointer passed in
+       * and return it
+       */
+      buf = bufp;
+
+      return buf;
+    }
+  }
+
+  /* Check for error conditions. */
+  if (ferror(cs->cs_file))
+    log_pri(LOG_ERR, "error while reading configuration stream: %s",
+      strerror(errno));
+
+  /* default return value
+   */
+  return NULL;
+}
+
+static cmd_rec *get_config_cmd(pool *ppool) {
+  char buf[TUNABLE_BUFFER_SIZE] = {'\0'}, *word = NULL;
+  cmd_rec *new_cmd = NULL;
+  pool *new_pool = NULL;
+  array_header *tarr = NULL;
+
+  while ((get_config_line(buf, TUNABLE_BUFFER_SIZE))) { 
+    char *bufp = buf;
+    
     /* Build a new pool for the command structure and array */
-    newpool = make_sub_pool(ppool);
-    newcmd = (cmd_rec*)pcalloc(newpool,sizeof(cmd_rec));
-    newcmd->pool = newpool;
-    tarr = make_array(newpool,4,sizeof(char**));
+    new_pool = make_sub_pool(ppool);
+    new_cmd = (cmd_rec *) pcalloc(new_pool, sizeof(cmd_rec));
+    new_cmd->pool = new_pool;
+    tarr = make_array(new_pool,4,sizeof(char**));
 
     /* Add each word to the array */
-    while((wrd = get_word(&cp)) != NULL) {
-      char *tmp = pstrdup(newpool, wrd);
+    while((word = get_word(&bufp)) != NULL) {
+      char *tmp = pstrdup(new_pool, word);
       
-      *((char**)push_array(tarr)) = tmp; /* pstrdup(newpool,wrd); */
-      newcmd->argc++;
+      *((char**)push_array(tarr)) = tmp; /* pstrdup(new_pool,word); */
+      new_cmd->argc++;
     }
 
     *((char**)push_array(tarr)) = NULL;
@@ -240,7 +372,7 @@ cmd_rec *get_config_cmd(pool *ppool, FILE *fp, int *line)
      * it will get purged when the command's pool is cleared
      */
 
-    newcmd->argv = (char**)tarr->elts;
+    new_cmd->argv = (char**)tarr->elts;
 
     /* Perform a fixup on configuration directives so that:
      * -argv[0]--  -argv[1]-- ----argv[2]-----
@@ -250,30 +382,29 @@ cmd_rec *get_config_cmd(pool *ppool, FILE *fp, int *line)
      * <Option>    /etc/adir  /etc/anotherdir
      */
 
-    if(newcmd->argc && *(newcmd->argv[0]) == '<') {
-      char *cp = newcmd->argv[newcmd->argc-1];
+    if(new_cmd->argc && *(new_cmd->argv[0]) == '<') {
+      char *cp = new_cmd->argv[new_cmd->argc-1];
 
-      if(*(cp + strlen(cp)-1) == '>' && newcmd->argc > 1) {
+      if(*(cp + strlen(cp)-1) == '>' && new_cmd->argc > 1) {
         if(!strcmp(cp,">")) {
-          newcmd->argv[newcmd->argc-1] = NULL;
-          newcmd->argc--;
+          new_cmd->argv[new_cmd->argc-1] = NULL;
+          new_cmd->argc--;
         } else
           *(cp + strlen(cp)-1) = '\0';
 
-        cp = newcmd->argv[0];
+        cp = new_cmd->argv[0];
         if(*(cp + strlen(cp)-1) != '>')
-          newcmd->argv[0] = pstrcat(newcmd->pool,cp,">",NULL);
+          new_cmd->argv[0] = pstrcat(new_cmd->pool,cp,">",NULL);
       }
     }
         
-    return newcmd;
+    return new_cmd;
   }
 
   return NULL;
 }
 
-void init_dyn_stacks(pool *p,config_rec *top)
-{
+static void init_dyn_stacks(pool *p, config_rec *top) {
   conf.sstack = make_array(p,1,sizeof(server_rec*));
   conf.curserver = (server_rec**)push_array(conf.sstack);
   *conf.curserver = main_server;
@@ -284,8 +415,11 @@ void init_dyn_stacks(pool *p,config_rec *top)
   *conf.curconfig = top;
 }
 
-void init_conf_stacks(void)
-{
+static void free_dyn_stacks(void) {
+  memset(&conf, '\0', sizeof(conf));
+}
+
+void init_conf_stacks(void) {
   pool *pool = make_sub_pool(permanent_pool);
 
   conf.tpool = pool;
@@ -297,15 +431,9 @@ void init_conf_stacks(void)
   *conf.curconfig = NULL;
 }
 
-void free_dyn_stacks(void)
-{
-  memset(&conf, 0, sizeof(conf));
-}
-
-void free_conf_stacks(void)
-{
+void free_conf_stacks(void) {
   destroy_pool(conf.tpool);
-  memset(&conf, 0, sizeof(conf));
+  memset(&conf, '\0', sizeof(conf));
 }
 
 /* Used by modules to start/end configuration sections */
@@ -702,34 +830,6 @@ config_rec *dir_match_path(pool *p, char *path)
   return res;
 }
 
-int dir_get_param(pool *pp,char *path,char *param)
-{
-  char *fullpath,*tmp;
-  pool *p;
-  config_rec *c;
-
-  p = make_sub_pool(pp);
-
-  if(*path != '/')
-    fullpath = pstrcat(p,session.cwd,"/",path,NULL);
-  else
-    fullpath = path;
-
-  if((tmp = dir_realpath(p,fullpath)) != NULL)
-    fullpath = tmp;
-
-  if(session.anon_root)
-    fullpath = pdircat(p,session.anon_root,fullpath,NULL);
-
-  c = dir_match_path(p,fullpath);
-
-  destroy_pool(p);
-
-  if(c)
-    return get_param_int(c->subset,param,FALSE);
-  return -1;
-}
-
 static int _dir_check_op(pool *p, xaset_t *c, int op, uid_t uid, gid_t gid,
     mode_t mode) {
   int i, res = 1, user_perms = 0;
@@ -868,16 +968,6 @@ int dir_check_op_mode(pool *p,char *path,int op,
   return res;  
 }
 
-int dir_check_op(pool *pp,char *path,int op)
-{
-  struct stat sbuf;
-
-  if(fs_stat(path,&sbuf) == -1)
-    return 1;
-
-  return dir_check_op_mode(pp,path,op,sbuf.st_uid,sbuf.st_gid,sbuf.st_mode);
-}
-
 static int _check_user_access(xaset_t *conf, char *name)
 {
   int ret = 0;
@@ -917,31 +1007,32 @@ static int _check_group_access(xaset_t *conf, char *name)
  * returns 0 if no match
  */
 
-int match_ip(p_in_addr_t *addr, char *name, const char *match)
-{
+int match_ip(p_in_addr_t *addr, char *name, const char *match) {
   char buf[1024];
   char *mask,*cp;
   int cidr_mode = 0, cidr_bits;
   p_in_addr_t cidr_addr;
   u_int_32 cidr_mask = 0;
 
-  if(!strcasecmp(match,"ALL"))
+  if (!strcasecmp(match, "ALL"))
     return 1;
 
-  if(!strcasecmp(match,"NONE"))
+  if (!strcasecmp(match, "NONE"))
     return -1;
 
-  memset(buf,0,sizeof(buf));
+  memset(buf, '\0', sizeof(buf));
   mask = buf;
 
-  if(*match == '.') {
+  if (*match == '.') {
     *mask++ = '*';
     *mask = '\0';
     sstrcat(buf, match, sizeof(buf));
-  } else if(*(match + strlen(match) - 1) == '.') {
+
+  } else if (*(match + strlen(match) - 1) == '.') {
     sstrcat(buf, match, sizeof(buf));
     sstrcat(buf, "*", sizeof(buf));
-  } else if((cp = strchr(match,'/')) != NULL) { /* check for CIDR notation */
+
+  } else if ((cp = strchr(match, '/')) != NULL) { /* check for CIDR notation */
     /* first portion of CIDR should be dotted quad, second portion
      * is netmask
      */
@@ -966,18 +1057,41 @@ int match_ip(p_in_addr_t *addr, char *name, const char *match)
     } else {
       return 0;
     }
+
   } else {
     sstrcat(buf, match, sizeof(buf));
   }
   
-  if(cidr_mode) {
+  if (cidr_mode) {
     if((addr->s_addr & htonl(cidr_mask)) == cidr_addr.s_addr)
       return 1;
+
   } else {
-    if(pr_fnmatch(buf, name, PR_FNM_NOESCAPE | PR_FNM_CASEFOLD) == 0 ||
-       pr_fnmatch(buf, inet_ntoa(*addr),
-		  PR_FNM_NOESCAPE | PR_FNM_CASEFOLD) == 0)
+    int fnm_flags = PR_FNM_NOESCAPE|PR_FNM_CASEFOLD;
+    pool *tmp_pool = make_sub_pool(permanent_pool);
+    p_in_addr_t *buf_addr = inet_getaddr(tmp_pool, buf);
+    char *buf_ascii = inet_ascii(tmp_pool, buf_addr),
+      *addr_ascii = inet_ascii(tmp_pool, addr);
+    
+    /* Note: do NOT use inet_ntoa(3) here, but rather use inet_ascii()
+     * wrapper function.  inet_ntoa(3)'s return value is a pointer to a
+     * buffer that is overwritten on subsequent calls.
+     */
+
+    log_debug(DEBUG6, "comparing addresses '%s' (%s) and '%s' (%s)",
+      buf, buf_ascii, name, addr_ascii);
+
+    if (!pr_fnmatch(buf, name, fnm_flags) ||
+        !pr_fnmatch(buf, addr_ascii, fnm_flags) ||
+        !pr_fnmatch(buf_ascii, addr_ascii, fnm_flags)) {
+      log_debug(DEBUG6, "addresses match");
+      destroy_pool(tmp_pool);
       return 1;
+
+    } else
+      log_debug(DEBUG6, "addresses do not match");
+
+    destroy_pool(tmp_pool);
   }
   
   return 0;
@@ -1340,7 +1454,7 @@ void build_dyn_config(pool *p,char *_path, struct stat *_sbuf, int recurse)
   FILE *fp;
   cmd_rec *cmd;
   xaset_t **set = NULL;
-  int isfile, line = 0, removed = 0;
+  int isfile, removed = 0;
 
   /* Switch through each directory, from "deepest" up looking for
    * new or updated .ftpaccess files
@@ -1428,14 +1542,20 @@ void build_dyn_config(pool *p,char *_path, struct stat *_sbuf, int recurse)
       /* File has been modified or not loaded yet */
       d->argv[0] = (void*)sbuf.st_mtime;
 
-      fp = fopen(dynpath,"r");
-      if(fp) {
+      if ((fp = fopen(dynpath, "r")) != NULL) {
+        config_stream_t *cs = NULL;
+
         removed = 0;
 
+        /* Push the configuration stream information onto the stack of
+         * configuration streams being parsed.
+         */
+        cs = push_config_stream(fp, 0);
+
         init_dyn_stacks(p,d);
         d->config_type = CONF_DYNDIR;
 
-        while((cmd = get_config_cmd(p, fp, &line)) != NULL) {
+        while((cmd = get_config_cmd(p)) != NULL) {
           if(cmd->argc) {
             conftable *c;
             char found = 0;
@@ -1445,7 +1565,8 @@ void build_dyn_config(pool *p,char *_path, struct stat *_sbuf, int recurse)
             cmd->config = *conf.curconfig;
               
             for(c = m_conftable; c->directive; c++) {
-              if(!strcasecmp(c->directive, cmd->argv[0])) {
+              if (!strcasecmp(c->directive, cmd->argv[0])) {
+                cmd->argv[0] = c->directive;
                 found++;
 
                 if((mr = call_module(c->m,c->handler,cmd)) != NULL) {
@@ -1464,7 +1585,8 @@ void build_dyn_config(pool *p,char *_path, struct stat *_sbuf, int recurse)
             if(!found)
               log_pri(LOG_WARNING,
                 "warning: unknown configuration directive '%s' on "
-                "line %d of '%s'.", cmd->argv[0], line, dynpath);
+                "line %d of '%s'.", cmd->argv[0], cs->cs_lineno,
+                dynpath);
 
           }
  
@@ -1478,6 +1600,11 @@ void build_dyn_config(pool *p,char *_path, struct stat *_sbuf, int recurse)
         free_dyn_stacks();
 
         _mergedown(*set,TRUE);
+
+        /* Pop this configuration stream from the stack.
+         */
+        pop_config_stream();
+
         fclose(fp);
       }
     }
@@ -1917,17 +2044,16 @@ static void _reorder_dirs(xaset_t *set, int mask)
       _reorder_dirs(c->subset,mask);
 }
 
-void debug_dump_config(xaset_t *s,char *indent)
-{
-  config_rec *c;
+static void debug_dump_config(xaset_t *s,char *indent) {
+  config_rec *c = NULL;
 
-  if(!indent)
+  if (!indent)
     indent = "";
 
-  for(c = (config_rec*)s->xas_list; c; c=c->next) {
-    log_debug(DEBUG5,"%s%s",indent,c->name);
-    if(c->subset)
-      debug_dump_config(c->subset,pstrcat(permanent_pool,indent," ",NULL));
+  for (c = (config_rec *) s->xas_list; c; c = c->next) {
+    log_debug(DEBUG5, "%s%s", indent, c->name);
+    if (c->subset)
+      debug_dump_config(c->subset, pstrcat(c->pool, indent," ", NULL));
   }
 }
 
@@ -2078,10 +2204,9 @@ void _copy_global_to_all(xaset_t *set)
       _copy_recur(&s->conf,s->pool,c,NULL);
 }
 
-void fixup_globals(void)
-{
-  server_rec *s,*smain;
-  config_rec *c,*cnext;
+static void fixup_globals(void) {
+  server_rec *s = NULL, *smain = NULL;
+  config_rec *c = NULL, *cnext = NULL;
 
   smain = (server_rec*)servers->xas_list;
   for(s = smain; s; s=s->next) {
@@ -2126,9 +2251,9 @@ void fixup_dirs(server_rec *s, int mask)
     if(c->config_type == CONF_ANON)
       _reorder_dirs(c->subset);
 */
-  log_debug(DEBUG5,"");
-  log_debug(DEBUG5,"Config for %s:",s->ServerName);
-  debug_dump_config(s->conf,NULL);
+  log_debug(DEBUG5, "%s", "");
+  log_debug(DEBUG5, "Config for %s:", s->ServerName);
+  debug_dump_config(s->conf, NULL);
 }
 
 config_rec *find_config_next(config_rec *prev, config_rec *c, int type,
@@ -2416,45 +2541,52 @@ config_rec *add_config_param(const char *name,int num,...)
   return c;
 }
 
-int parse_config_file(const char *fname)
-{
-  FILE *fp;
-  cmd_rec *cmd;
+int parse_config_file(const char *fname) {
+  FILE *fp = NULL;
+  config_stream_t *cs = NULL;
+  cmd_rec *cmd = NULL;
+  modret_t *mr = NULL;
   pool *tmp_pool = make_sub_pool(permanent_pool);
-  modret_t *mr;
-  int line = 0;
  
-  fp = pfopen(tmp_pool,fname,"r");
+  if ((fp = pfopen(tmp_pool, fname, "r")) == NULL) {
+    destroy_pool(tmp_pool);
+    return -1;
+  }
 
-  if(!fp) { destroy_pool(tmp_pool); return -1; }
-  
-  while((cmd = get_config_cmd(tmp_pool, fp, &line)) != NULL) {
-    if(cmd->argc) {
+  /* Push the configuration stream information onto the stack of
+   * configuration streams.
+   */
+  cs = push_config_stream(fp, 0);
+ 
+  while ((cmd = get_config_cmd(tmp_pool)) != NULL) {
+    if (cmd->argc) {
       conftable *c;
       char found = 0;
 
       cmd->server = *conf.curserver;
       cmd->config = *conf.curconfig;
 
-      for(c = m_conftable; c->directive; c++)
-        if(!strcasecmp(c->directive,cmd->argv[0])) {
+      for (c = m_conftable; c->directive; c++)
+        if (!strcasecmp(c->directive, cmd->argv[0])) {
+          cmd->argv[0] = c->directive;
           ++found;
-          if((mr = call_module(c->m,c->handler,cmd)) != NULL) {
-            if(MODRET_ISERROR(mr)) {
-	            log_pri(LOG_ERR,"Fatal: %s",MODRET_ERRMSG(mr));
-	            exit(1);
+
+          if ((mr = call_module(c->m, c->handler, cmd)) != NULL) {
+            if (MODRET_ISERROR(mr)) {
+              log_pri(LOG_ERR, "Fatal: %s", MODRET_ERRMSG(mr));
+              exit(1);
 	    }
           }
 
-	  if(MODRET_ISDECLINED(mr))
+	  if (MODRET_ISDECLINED(mr))
 	    found--;
 
           destroy_pool(cmd->tmp_pool);
         }
 
-       if(!found) {
-         log_pri(LOG_ERR,"Fatal: unknown configuration directive '%s' on line %d of '%s'.",
-                 cmd->argv[0], line, fname);
+       if (!found) {
+         log_pri(LOG_ERR, "Fatal: unknown configuration directive '%s' on "
+           "line %d of '%s'.", cmd->argv[0], cs->cs_lineno, fname);
          exit(1);
        }
     }
@@ -2462,6 +2594,10 @@ int parse_config_file(const char *fname)
     destroy_pool(cmd->pool);
   }
 
+  /* Pop this configuration stream from the stack.
+   */
+  pop_config_stream();
+
   pfclose(tmp_pool,fp);
   destroy_pool(tmp_pool);
   return 0;
@@ -2542,6 +2678,17 @@ void init_config(void) {
     global_config_pool = NULL;
   }
 
+  if (servers) {
+    server_rec *s, *s_next;
+
+    /* Free the old configuration completely */
+    for (s = (server_rec *) servers->xas_list; s; s = s_next) {
+      s_next = s->next;
+      destroy_pool(s->pool);
+    }
+    destroy_pool(servers->mempool);
+  }
+
   servers = xaset_create(pool,NULL);
 
   pool = make_sub_pool(permanent_pool);
