@@ -1,7 +1,8 @@
 /*
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
- * Copyright (C) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 2001, 2002 The ProFTPD Project team
  *  
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,30 +48,28 @@ extern server_rec *main_server;
  */
 
 static pool *inet_pool = NULL;
-static int tcp_proto = 6;		/* Generally, this is "tcp" */
+static int tcp_proto = IPPROTO_TCP;
 static int reverse_dns = 1;		/* Use reverse dns? */
 static int inet_errno = 0;		/* Holds errno */
 
 /* Cleanup for inet_pool
  */
 
-static void _inet_pool_cleanup(void *ignore)
-{
+static void inet_pool_cleanup_cb(void *ignore) {
   inet_pool = NULL;
 }
 
 /* Create the private inet pool
  */
 
-static void _create_inet_pool(void)
-{
+static void _create_inet_pool(void) {
   inet_pool = make_sub_pool(permanent_pool);
 
   /* Necessary to register a cleanup, in case the whole process
    * execs and we lose our pool.
    */
 
-  register_cleanup(inet_pool,NULL,_inet_pool_cleanup,_inet_pool_cleanup);
+  register_cleanup(inet_pool, NULL, inet_pool_cleanup_cb, inet_pool_cleanup_cb);
 }
 
 /*
@@ -78,12 +77,11 @@ static void _create_inet_pool(void)
  * to free up memory.
  */
 
-void clear_inet_pool(void)
-{
-  if(!inet_pool)		/* Sanity check */
+void clear_inet_pool(void) {
+  if (!inet_pool)		/* Sanity check */
     return;
 
-  kill_cleanup(inet_pool,NULL,_inet_pool_cleanup);
+  unregister_cleanup(inet_pool, NULL, inet_pool_cleanup_cb);
   destroy_pool(inet_pool);
   inet_pool = NULL;
 }
@@ -269,7 +267,7 @@ char *inet_getname(pool *pool, p_in_addr_t *addr)
   return inet_validate(res);
 }
 
-static void _conn_cleanup(void *cv) { 
+static void conn_cleanup_cb(void *cv) { 
   conn_t *c = (conn_t*)cv;
 
   if(c->inf)
@@ -320,7 +318,7 @@ conn_t *inet_copy_connection(pool *p, conn_t *c)
   if(c->iplist)
     res->iplist = copy_array(subpool,c->iplist);
 
-  register_cleanup(res->pool,(void*)res,_conn_cleanup,_conn_cleanup);
+  register_cleanup(res->pool, (void *) res, conn_cleanup_cb, conn_cleanup_cb);
   return res;
 }
 
@@ -519,7 +517,8 @@ static conn_t *inet_initialize_connection(pool *p, xaset_t *servers, int fd,
       res = bind(fd, (struct sockaddr *) &servaddr, sizeof(servaddr));
       hold_errno = errno;
 
-      if(res == -1 && errno == EINTR) {
+      if (res == -1 && errno == EINTR) {
+        pr_handle_signals();
 	i++;
 	continue;
       }
@@ -580,7 +579,7 @@ static conn_t *inet_initialize_connection(pool *p, xaset_t *servers, int fd,
   }
   
   c->listen_fd = fd;
-  register_cleanup(c->pool, (void *) c, _conn_cleanup, _conn_cleanup);
+  register_cleanup(c->pool, (void *) c, conn_cleanup_cb, conn_cleanup_cb);
   
   return c;
 }
@@ -669,7 +668,7 @@ void inet_close(pool *pool, conn_t *c)
   /* It's not necessary to close the fds or schedule IOFILEs for
    * removal, because the creator of the connection (either
    * inet_create_connection() or inet_copy_connection() will
-   * have registered a pool cleanup handler (_conn_cleanup())
+   * have registered a pool cleanup handler (conn_cleanup_cb())
    * which will do all this for us.  Simply destroy the pool
    * and all the dirty work gets done. :-)
    */
@@ -677,6 +676,25 @@ void inet_close(pool *pool, conn_t *c)
   destroy_pool(c->pool);
 }
 
+/* perform shutdown/read on streams */
+void inet_lingering_close(pool *pool, conn_t *c, long linger) {
+  inet_setblock(pool, c);
+
+  if (c->outf)
+    io_lingering_close(c->outf, linger);
+
+  /* Only close the input stream if it is actually a different stream than
+   * the output stream.
+   */
+  if (c->inf != c->outf)
+    io_lingering_close(c->inf, linger);
+
+  c->outf = NULL;
+  c->inf = NULL;
+
+  destroy_pool(c->pool);
+}
+
 int inet_set_proto_options(pool *pool, conn_t *c,
 			int nodelay,
 			int lowdelay,
@@ -730,15 +748,10 @@ int inet_set_proto_options(pool *pool, conn_t *c,
 int inet_setoptions(pool *pool, conn_t *c, int rcvbuf, int sndbuf)
 {
   int no_keep_alive = 0;
-  struct linger li;
   int len,crcvbuf,csndbuf;
 
-  li.l_onoff = 0;
-  li.l_linger = 0;
-
   if(c->wfd != -1) {
     setsockopt(c->wfd,SOL_SOCKET,SO_KEEPALIVE,(void*)&no_keep_alive,sizeof(int));
-    setsockopt(c->wfd,SOL_SOCKET,SO_LINGER,(void*)&li,sizeof(li));
 
     /* Linux and "most" newer networking OSes probably use a highly
      * adaptive window size system, which generally wouldn't require
@@ -758,7 +771,6 @@ int inet_setoptions(pool *pool, conn_t *c, int rcvbuf, int sndbuf)
 
   if(c->rfd != -1) {
     setsockopt(c->rfd,SOL_SOCKET,SO_KEEPALIVE,(void*)&no_keep_alive,sizeof(int));
-    setsockopt(c->wfd,SOL_SOCKET,SO_LINGER,(void*)&li,sizeof(li));
 
     len = sizeof(crcvbuf);
     getsockopt(c->rfd,SOL_SOCKET,SO_RCVBUF,(void*)&crcvbuf,&len);
@@ -875,13 +887,16 @@ int inet_listen(pool *pool, conn_t *c, int backlog)
   if(!c || c->mode == CM_LISTEN)
     return -1;
 
-  while(1)
-    if(listen(c->listen_fd,backlog) == -1) {
-      if(errno == EINTR)
+  while (1)
+    if (listen(c->listen_fd, backlog) == -1) {
+      if (errno == EINTR) {
+        pr_handle_signals();
         continue; 
-      log_pri(LOG_ERR,"listen() failed in inet_listen(): %s",
-              strerror(errno));
+      }
+
+      log_pri(LOG_ERR,"listen() failed in inet_listen(): %s", strerror(errno));
       end_login(1);
+
     } else
       break;
   
@@ -914,10 +929,17 @@ int inet_connect(pool *pool, conn_t *c, p_in_addr_t *addr, int port)
 
   c->mode = CM_CONNECT;
 
-  while( (ret = connect(c->listen_fd,(struct sockaddr*)&remaddr,sizeof(remaddr))) == -1 &&
-         errno == EINTR ) ;
+  while (TRUE) {
+    if ((ret = connect(c->listen_fd, (struct sockaddr *) &remaddr,
+        sizeof(remaddr))) == -1 && errno == EINTR) {
+      pr_handle_signals();
+      continue;
 
-  if(ret == -1) {
+    } else
+      break;
+  }
+
+  if (ret == -1) {
     c->mode = CM_ERROR;
     c->xerrno = errno;
     return -1;
@@ -1010,31 +1032,33 @@ int inet_accept_nowait(pool *pool, conn_t *c)
  * it, or NULL upon error.
  */
 conn_t *inet_accept(pool *pool, conn_t *d, conn_t *c, int rfd, int wfd,
-		    int resolve) {
+    unsigned char resolve) {
   conn_t *res = NULL;
-  int newfd = -1, allow_foreign_addr = -1;
+  unsigned char *allow_foreign_addr = NULL;
+  int newfd = -1;
   struct sockaddr_in addr;
   int addrlen = sizeof(addr);
   
   d->mode = CM_ACCEPT;
-  allow_foreign_addr = get_param_int(TOPLEVEL_CONF, "AllowForeignAddress",
-				     FALSE);
+
+  allow_foreign_addr = get_param_ptr(TOPLEVEL_CONF,
+    "AllowForeignAddress", FALSE);
   
   for(;;) {
-    if((newfd = accept(d->listen_fd, (struct sockaddr *) &addr,
-		       &addrlen)) != -1) {
-      if((allow_foreign_addr != 1) &&
+    if ((newfd = accept(d->listen_fd, (struct sockaddr *) &addr,
+        &addrlen)) != -1) {
+      if ((allow_foreign_addr && *allow_foreign_addr == FALSE) &&
 	 (getpeername(newfd, (struct sockaddr *) &addr, &addrlen) != -1)) {
-	if(addr.sin_addr.s_addr != c->remote_ipaddr->s_addr) {
-	  log_pri(LOG_NOTICE,
-		  "SECURITY VIOLATION: Passive connection from %s rejected.",
-		  inet_ntoa(addr.sin_addr));
+	if (addr.sin_addr.s_addr != c->remote_ipaddr->s_addr) {
+	  log_pri(LOG_NOTICE, "SECURITY VIOLATION: Passive connection from "
+            "%s rejected.", inet_ntoa(addr.sin_addr));
 	  close(newfd);
 	  continue;
 	}
       }
       d->mode = CM_OPEN;
       res = inet_openrw(pool, d, NULL, newfd, rfd, wfd, resolve);
+
     } else {
       d->mode = CM_ERROR;
       d->xerrno = errno;
@@ -1243,8 +1267,14 @@ void init_inet(void)
 {
   struct protoent *pr;
 
+#ifdef HAVE_SETPROTOENT
   setprotoent(FALSE);
+#endif
+
   if((pr = getprotobyname("tcp")) != NULL)
     tcp_proto = pr->p_proto;
+
+#ifdef HAVE_ENDPROTOENT
   endprotoent();
+#endif
 }
