@@ -1,7 +1,8 @@
 /*
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
- * Copyright (C) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 2001, 2002 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +27,7 @@
 /*
  * ProFTPD logging support.
  *
- * $Id: log.c,v 1.35 2002/05/21 20:47:22 castaglia Exp $
+ * $Id: log.c,v 1.42 2002/08/13 22:52:00 castaglia Exp $
  */
 
 /* History Log:
@@ -102,13 +103,15 @@ int log_open_xfer(const char *fn) {
     return 0;
   }
 
-  if (xferfd == -1)
+  if (xferfd == -1) {
+    log_debug(DEBUG6, "opening TransferLog '%s'", fn);
     log_openfile(fn, &xferfd, LOG_XFER_MODE);
+  }
 
   return xferfd;
 }
 
-int log_xfer(int xfertime, char *remhost, unsigned long fsize, char *fname,
+int log_xfer(int xfertime, char *remhost, off_t fsize, char *fname,
     char xfertype, char direction, char access, char *user, char abort_flag) {
 
   char buf[LOGBUFFER_SIZE] = {'\0'}, fbuf[LOGBUFFER_SIZE] = {'\0'};
@@ -121,7 +124,7 @@ int log_xfer(int xfertime, char *remhost, unsigned long fsize, char *fname,
     fbuf[i] = (isspace(fname[i]) || iscntrl(fname[i])) ? '_' : fname[i];
   fbuf[i] = '\0';
 
-  snprintf(buf, sizeof(buf), "%s %d %s %lu %s %c _ %c %c %s ftp %c %s %c\n",
+  snprintf(buf, sizeof(buf), "%s %d %s %" PR_LU " %s %c _ %c %c %s ftp %c %s %c\n",
     fmt_time(time(NULL)), xfertime, remhost, fsize, fbuf, xfertype, direction,
     access, user, session.ident_lookups == TRUE ? '1' : '0',
     (session.ident_lookups == TRUE && strcmp(session.ident_user,
@@ -251,7 +254,10 @@ int log_open_run(pid_t mpid, int trunc, int allow_update)
   /* Attempt to read header */
   i = read(runfd, &hdr, sizeof(hdr));
 
-  if(i <= 0) {
+  /* Note: AIX seems to needs this st_size check.  Weird.  It won't (read:
+   * _shouldn't_) hurt any other platforms.
+   */
+  if (i <= 0 || sbuf.st_size == 0) {
     char buf[sizeof(logrun_t)] = {'\0'};
     runsize = sizeof(logrun_t);
     
@@ -266,7 +272,7 @@ int log_open_run(pid_t mpid, int trunc, int allow_update)
     return runfd;
   }
 
-  if(i < sizeof(hdr)) {
+  if (i < sizeof(hdr)) {
     /* File is corrupt, etc, silently rm it */
     if(allow_update) {
       log_rm_run();
@@ -637,8 +643,10 @@ int log_wtmp(char *line, char *name, char *host, p_in_addr_t *ip)
       utx.ut_type = USER_PROCESS;
     else
       utx.ut_type = DEAD_PROCESS;
+#ifdef HAVE_UT_UT_EXIT
     utx.ut_exit.e_termination = 0;
     utx.ut_exit.e_exit = 0;
+#endif /* HAVE_UT_UT_EXIT */
     if(write(fdx,(char*)&utx,sizeof(utx)) != sizeof(utx))
       ftruncate(fdx, buf.st_size);
   } else {
@@ -661,8 +669,10 @@ int log_wtmp(char *line, char *name, char *host, p_in_addr_t *ip)
       memcpy(&ut.ut_addr,ip,sizeof(ut.ut_addr));
 #else
     sstrncpy(ut.ut_id,"ftp",sizeof(ut.ut_id));
+#ifdef HAVE_UT_UT_EXIT
     ut.ut_exit.e_termination = 0;
     ut.ut_exit.e_exit = 0;
+#endif /* HAVE_UT_UT_EXIT */
 #endif
     sstrncpy(ut.ut_line,line,sizeof(ut.ut_line));
     if(name && *name)
@@ -821,8 +831,7 @@ void log_discard(void)
   syslog_discard = TRUE;
 }
 
-void log(int priority, int f, char *s)
-{
+static void log(int priority, int f, char *s) {
   int maxpriority;
   char serverinfo[1024];
   
@@ -840,15 +849,15 @@ void log(int priority, int f, char *s)
     }
   }
   
-  if(logstderr) {
+  if (logstderr) {
     fprintf(stderr, "%s - %s\n", serverinfo, s);
     return;
   }
   
-  if(syslog_discard)
+  if (syslog_discard)
     return;
   
-  if(syslog_fd != -1) {
+  if (syslog_fd != -1) {
     char buf[LOGBUFFER_SIZE] = {'\0'};
     time_t tt = time(NULL);
     struct tm *t;
@@ -857,7 +866,7 @@ void log(int priority, int f, char *s)
     strftime(buf, sizeof(buf), "%b %d %H:%M:%S ", t);
     buf[sizeof(buf) - 1] = '\0';
     
-    if(serverinfo && *serverinfo) {
+    if (*serverinfo) {
       snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf),
 	       "%s proftpd[%u] %s: %s\n", syslog_hostname,
 	       (unsigned int) getpid(), serverinfo, s);
@@ -872,37 +881,37 @@ void log(int priority, int f, char *s)
     return;
   }
   
-  if(set_facility != -1)
+  if (set_facility != -1)
     f = set_facility;
 
-  if(f != facility || !syslog_open)
+  if (f != facility || !syslog_open)
     openlog("proftpd", LOG_NDELAY | LOG_PID, f);
   
-  if((maxpriority = get_param_int(main_server->conf, "SyslogLevel",
+  if ((maxpriority = get_param_int(main_server->conf, "SyslogLevel",
 				  FALSE)) != -1)
-    if(priority > maxpriority)
+    if (priority > maxpriority)
       return;
   
-  if(serverinfo && *serverinfo)
+  if (*serverinfo)
     syslog(priority, "%s - %s\n", serverinfo, s);
   else
     syslog(priority, "%s\n", s);
   
-  if(!syslog_open)
+  if (!syslog_open)
     closelog();
-  else if(f != facility)
+  else if (f != facility)
     openlog("proftpd", LOG_NDELAY | LOG_PID, facility);
 }
 
-void log_pri(int priority, char *fmt, ...)
-{
+void log_pri(int priority, char *fmt, ...) {
   char buf[LOGBUFFER_SIZE] = {'\0'};
   va_list msg;
   
-  va_start(msg,fmt);
+  va_start(msg, fmt);
   vsnprintf(buf, sizeof(buf), fmt, msg);
   va_end(msg);
-  
+ 
+  /* Always make sure the buffer is NUL-terminated. */ 
   buf[sizeof(buf) - 1] = '\0';
   
   log(priority, facility, buf);
@@ -912,15 +921,15 @@ void log_pri(int priority, char *fmt, ...)
  * facility (presumable it doesn't need to be seen by everyone
  */
 
-void log_auth(int priority, char *fmt, ...)
-{
+void log_auth(int priority, char *fmt, ...) {
   char buf[LOGBUFFER_SIZE] = {'\0'};
   va_list msg;
 
-  va_start(msg,fmt);
+  va_start(msg, fmt);
   vsnprintf(buf, sizeof(buf), fmt, msg);
   va_end(msg);
 
+  /* Always make sure the buffer is NUL-terminated. */
   buf[sizeof(buf) - 1] = '\0';
 
   log(priority, LOG_AUTHPRIV, buf);
@@ -948,19 +957,19 @@ int log_setdebuglevel(int level)
   return old_level;
 }
 
-void log_debug(int level,char *str,...)
-{
-  char buf[LOGBUFFER_SIZE];
+void log_debug(int level, char *fmt, ...) {
+  char buf[LOGBUFFER_SIZE] = {'\0'};
   va_list msg;
 
-  if(debug_level < level)
+  if (debug_level < level)
     return;
 
-  memset(buf,'\0',sizeof(buf));
-  va_start(msg, str);
-  vsnprintf(buf, sizeof(buf), str, msg);
+  memset(buf, '\0', sizeof(buf));
+  va_start(msg, fmt);
+  vsnprintf(buf, sizeof(buf), fmt, msg);
   va_end(msg);
 
+  /* Always make sure the buffer is NUL-terminated. */
   buf[sizeof(buf) - 1] = '\0';
 
   log(LOG_DEBUG, facility, buf);
