@@ -1,7 +1,8 @@
 /*
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
- * Copyright (C) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 2001, 2002 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,8 +32,7 @@
 
 /* Manage free storage blocks */
 
-union align
-{
+union align {
   char *cp;
   void (*f)();
   long l;
@@ -42,8 +42,7 @@ union align
 
 #define CLICK_SZ (sizeof(union align))
 
-union block_hdr
-{
+union block_hdr {
   union align a;
 
   /* Actual header */
@@ -58,14 +57,13 @@ union block_hdr
 union block_hdr *block_freelist = NULL;
 
 /* Statistics */
-unsigned int stat_malloc = 0;	/* incr when malloc required */
-unsigned int stat_freehit = 0;	/* incr when freelist used */
+static unsigned int stat_malloc = 0;	/* incr when malloc required */
+static unsigned int stat_freehit = 0;	/* incr when freelist used */
 
 /* Lowest level memory allocation functions
  */
 
-static void *null_alloc(size_t size)
-{
+static void *null_alloc(size_t size) {
   void *ret = 0;
 
   if(size == 0)
@@ -78,8 +76,7 @@ static void *null_alloc(size_t size)
   return ret;
 }
 
-void *xmalloc(size_t size)
-{
+void *xmalloc(size_t size) {
   void *ret;
 
   ret = malloc(size);
@@ -88,8 +85,7 @@ void *xmalloc(size_t size)
   return ret;
 }
 
-void *xcalloc(size_t num, size_t size)
-{
+void *xcalloc(size_t num, size_t size) {
   void *ret;
 
   ret = calloc(num,size);
@@ -98,8 +94,7 @@ void *xcalloc(size_t num, size_t size)
   return ret;
 }
 
-void *xrealloc(void *p, size_t size)
-{
+void *xrealloc(void *p, size_t size) {
   if(p == 0)
     return xmalloc(size);
   p = realloc(p,size);
@@ -109,11 +104,10 @@ void *xrealloc(void *p, size_t size)
 }
 
 /* Grab a completely new block from the system pool.  Relies on malloc()
- * to return truely aligned memory.
+ * to return truly aligned memory.
  */
 
-union block_hdr *malloc_block(int size)
-{
+static union block_hdr *malloc_block(int size) {
   union block_hdr *blok =
     (union block_hdr*)xmalloc(size + sizeof(union block_hdr));
 
@@ -124,8 +118,7 @@ union block_hdr *malloc_block(int size)
   return blok;
 }
 
-void chk_on_blk_list(union block_hdr *blok, union block_hdr *free_blk)
-{
+static void chk_on_blk_list(union block_hdr *blok, union block_hdr *free_blk) {
   /* Debug code */
 
   while(free_blk) {
@@ -140,8 +133,7 @@ void chk_on_blk_list(union block_hdr *blok, union block_hdr *free_blk)
 
 /* Free a chain of blocks -- _must_ call with alarms blocked. */
 
-void free_blocks(union block_hdr *blok)
-{
+static void free_blocks(union block_hdr *blok) {
   /* Puts new blocks at head of block list, point next pointer of
    * last block in chain to free blocks we already had.
    */
@@ -170,8 +162,7 @@ void free_blocks(union block_hdr *blok)
  * a new one.  *BLOCK ALARMS BEFORE CALLING*
  */
 
-union block_hdr *new_block(int min_size)
-{
+static union block_hdr *new_block(int min_size) {
   int biggest = 0;
   union block_hdr **lastptr = &block_freelist;
   union block_hdr *blok = block_freelist;
@@ -200,9 +191,8 @@ union block_hdr *new_block(int min_size)
 
 /* accounting */
 
-long bytes_in_block_list(union block_hdr *blok)
-{
-  long size = 0;
+static unsigned long bytes_in_block_list(union block_hdr *blok) {
+  unsigned long size = 0;
 
   while(blok) {
     size += blok->h.endp - (char*)(blok+1);
@@ -265,19 +255,17 @@ static long __walk_pools(pool *p, int level)
     total += bytes_in_block_list(p->first);
     if(level == 0) {
       if(p->symbol)
-        log_pri(LOG_NOTICE,"(%s)0x%08x bytes",
-			&p->symbol,
+        log_pri(LOG_NOTICE,"(%s)0x%08lx bytes", &p->symbol,
 			bytes_in_block_list(p->first));
       else
-        log_pri(LOG_NOTICE,"0x%08x bytes",
+        log_pri(LOG_NOTICE,"0x%08lx bytes",
 	  	      	bytes_in_block_list(p->first));
     } else {
       if(p->symbol)
-	log_pri(LOG_NOTICE,"%s(%s)\\- 0x%08x bytes",_levelpad,
-			&p->symbol,
+	log_pri(LOG_NOTICE,"%s(%s)\\- 0x%08lx bytes",_levelpad, &p->symbol,
 			bytes_in_block_list(p->first));
       else
-        log_pri(LOG_NOTICE,"%s\\- 0x%08x bytes",_levelpad,
+        log_pri(LOG_NOTICE,"%s\\- 0x%08lx bytes",_levelpad,
               bytes_in_block_list(p->first));
     }
     
@@ -289,23 +277,21 @@ static long __walk_pools(pool *p, int level)
   return total;
 }
 
-void debug_pool_info(void)
-{
-  if(block_freelist)
-    log_pri(LOG_NOTICE,"Free block list: 0x%08x bytes",
-            bytes_in_block_list(block_freelist));
+static void debug_pool_info(void) {
+  if (block_freelist)
+    log_pri(LOG_NOTICE, "Free block list: 0x%08lx bytes",
+      bytes_in_block_list(block_freelist));
   else
-    log_pri(LOG_NOTICE,"Free block list: EMPTY");
+    log_pri(LOG_NOTICE, "Free block list: EMPTY");
 
-  log_pri(LOG_NOTICE,"%u count blocks malloc'd.",stat_malloc);
-  log_pri(LOG_NOTICE,"%u count blocks reused.",stat_freehit); 
+  log_pri(LOG_NOTICE, "%u count blocks malloc'd.", stat_malloc);
+  log_pri(LOG_NOTICE, "%u count blocks reused.", stat_freehit); 
 }
 
-void debug_walk_pools(void)
-{
-  log_pri(LOG_NOTICE,"Memory pool allocation:");
-  log_pri(LOG_NOTICE,"Total 0x%08x bytes allocated",
-          __walk_pools(permanent_pool,0));
+void debug_walk_pools(void) {
+  log_pri(LOG_NOTICE, "Memory pool allocation:");
+  log_pri(LOG_NOTICE, "Total 0x%08lx bytes allocated",
+    __walk_pools(permanent_pool, 0));
   debug_pool_info();
 }
 
@@ -339,20 +325,6 @@ struct pool *make_named_sub_pool(struct pool *p, const char *symbol)
   blok->h.first_avail += POOL_HDR_BYTES;
   memset((char *) new_pool, 0, sizeof(struct pool));
 
-#if 0 /* This is simply unused, and will be removed in 1.3 - MacGyver */  
-  if(symbol) {
-    /* This could be questionable... - MacGyver
-     */
-    sstrncpy(&new_pool->symbol, symbol, strlen(&new_pool->symbol));
-    
-    /* Alignment issues on Sparc, SGI, and probably other hardware,
-     * demand this.
-     */
-    blok->h.first_avail += (strlen(symbol) / POOL_HDR_BYTES + 1) *
-      POOL_HDR_BYTES;
-  }
-#endif
-
   new_pool->free_first_avail = blok->h.first_avail;
   new_pool->first = new_pool->last = blok;
   
@@ -369,9 +341,8 @@ struct pool *make_named_sub_pool(struct pool *p, const char *symbol)
   return new_pool;
 }
 
-struct pool *make_sub_pool(struct pool *p)
-{
-  return make_named_sub_pool(p,NULL);
+struct pool *make_sub_pool(struct pool *p) {
+  return make_named_sub_pool(p, NULL);
 }
 
 /* Initialize the pool system by creating the base permanent_pool. */
@@ -382,20 +353,21 @@ void init_alloc(void) {
 
 static void clear_pool(struct pool *p)
 {
-
-  if(!p)
-    return;			/* Sanity check */
+  /* Sanity check. */
+  if (! p)
+    return;
 
   block_alarms();
 
-  run_cleanups(p->cleanups);	 	p->cleanups = NULL;
+  run_cleanups(p->cleanups);
+  p->cleanups = NULL;
 
-  while(p->sub_pools)
+  while (p->sub_pools)
     destroy_pool(p->sub_pools);
-
   p->sub_pools = NULL;
 
-  free_blocks(p->first->h.next);	p->first->h.next = NULL;
+  free_blocks(p->first->h.next);
+  p->first->h.next = NULL;
 
   p->last = p->first;
   p->first->h.first_avail = p->free_first_avail;
@@ -407,14 +379,13 @@ void destroy_pool(pool *p)
 {
   block_alarms();
 
-  if(p->parent) {
-    if(p->parent->sub_pools == p) p->parent->sub_pools = p->sub_next;
-    if(p->sub_prev) p->sub_prev->sub_next = p->sub_next;
-    if(p->sub_next) p->sub_next->sub_prev = p->sub_prev;
+  if (p->parent) {
+    if (p->parent->sub_pools == p) p->parent->sub_pools = p->sub_next;
+    if (p->sub_prev) p->sub_prev->sub_next = p->sub_next;
+    if (p->sub_next) p->sub_next->sub_prev = p->sub_prev;
   }
 
   clear_pool(p);
-
   free_blocks(p->first);
 
   unblock_alarms();
@@ -468,8 +439,8 @@ void *palloc(struct pool *p, int reqsize)
 
 void *pcalloc(struct pool *p, int size)
 {
-  void *res = palloc(p,size);
-  memset(res,'\0',size);
+  void *res = palloc(p, size);
+  memset(res, '\0', size);
   return res;
 }
 
@@ -477,7 +448,7 @@ char *pstrdup(struct pool *p, const char *s)
 {
   char *res;
 
-  if(!s)
+  if (!s)
     return NULL;
 
   res = palloc(p, strlen(s) + 1);
@@ -489,11 +460,10 @@ char *pstrndup(struct pool *p, const char *s, int n)
 {
   char *res;
 
-  if(!s)
+  if (!s)
     return NULL;
 
   res = palloc(p, n + 1);
-
   sstrncpy(res, s, n + 1);
   return res;
 }
@@ -678,31 +648,35 @@ array_header *append_arrays(pool *p,
  * Generic cleanups
  */
 
-struct cleanup {
+typedef struct cleanup {
   void *data;
-  void (*plain_cleanup)(void*);
-  void (*child_cleanup)(void*);
+  void (*plain_cleanup_cb)(void *);
+  void (*child_cleanup_cb)(void *);
   struct cleanup *next;
-};
+} cleanup_t;
 
-void register_cleanup(pool *p, void *data, void (*plain_cleanup)(void*),
-                      void (*child_cleanup)(void*))
-{
-  struct cleanup *c = (struct cleanup*)palloc(p, sizeof(struct cleanup));
+void register_cleanup(pool *p, void *data, void (*plain_cleanup_cb)(void*),
+    void (*child_cleanup_cb)(void *)) {
+  cleanup_t *c = pcalloc(p, sizeof(cleanup_t));
   c->data = data;
-  c->plain_cleanup = plain_cleanup;
-  c->child_cleanup = child_cleanup;
+  c->plain_cleanup_cb = plain_cleanup_cb;
+  c->child_cleanup_cb = child_cleanup_cb;
+
+  /* Add this cleanup to the given pool's list of cleanups. */
   c->next = p->cleanups;
   p->cleanups = c;
 }
 
-void kill_cleanup(pool *p, void *data, void (*cleanup)(void*))
-{
-  struct cleanup *c = p->cleanups;
-  struct cleanup **lastp = &p->cleanups;
+void unregister_cleanup(pool *p, void *data, void (*cleanup_cb)(void *)) {
+  cleanup_t *c = p->cleanups;
+  cleanup_t **lastp = &p->cleanups;
 
-  while(c) {
-    if(c->data == data && c->plain_cleanup == cleanup) {
+  while (c) {
+    if (c->data == data && c->plain_cleanup_cb == cleanup_cb) {
+
+      /* Remove the given cleanup by pointing the previous next pointer to
+       * the matching cleanup's next pointer.
+       */
       *lastp = c->next;
       break;
     }
@@ -712,32 +686,40 @@ void kill_cleanup(pool *p, void *data, void (*cleanup)(void*))
   }
 }
 
-void run_cleanup(pool *p, void *data, void (*cleanup)(void*))
-{
+/* NOTE: unused. */
+#if 0
+void run_cleanup(pool *p, void *data, void (*cleanup_cb)(void *)) {
   block_alarms();
-  (*cleanup)(data);
-  kill_cleanup(p,data,cleanup);
+
+  /* Run the given cleanup callback. */
+  (*cleanup_cb)(data);
+
+  /* Remove it. */
+  unregister_cleanup(p, data, cleanup_cb);
+
   unblock_alarms();
 }
+#endif
 
-static void run_cleanups(struct cleanup *c)
-{
+static void run_cleanups(cleanup_t *c) {
   while(c) {
-    (*c->plain_cleanup)(c->data);
+    (*c->plain_cleanup_cb)(c->data);
     c = c->next;
   }
 }
 
-static void run_child_cleanups(struct cleanup *c)
-{
-  while(c) {
-    (*c->child_cleanup)(c->data);
+/* NOTE: these cleanup routines are currently unused.
+ * 2002-07-24
+ */
+#if 0
+static void run_child_cleanups(cleanup_t *c) {
+  while (c) {
+    (*c->child_cleanup_cb)(c->data);
     c = c ->next;
   }
 }
 
-static void cleanup_pool_for_exec(pool *p)
-{
+static void cleanup_pool_for_exec(pool *p) {
   run_child_cleanups(p->cleanups);
   p->cleanups = NULL;
 
@@ -745,48 +727,41 @@ static void cleanup_pool_for_exec(pool *p)
     cleanup_pool_for_exec(p);
 }
 
-void cleanup_for_exec(void)
-{
+void cleanup_for_exec(void) {
   block_alarms();
   cleanup_pool_for_exec(permanent_pool);
   unblock_alarms();
 }
+#endif
 
 /*
  * Files and file descriptors
  */
 
-static void fd_cleanup(void *fdv) { close ((int)fdv); }
-
-void note_cleanups_for_fd(pool *p, int fd)
-{
-  register_cleanup(p,(void*)fd,fd_cleanup,fd_cleanup);
+static void fd_cleanup_cb(void *fdv) {
+  close((int)fdv);
 }
 
-void kill_cleanups_for_fd(pool *p, int fd)
-{
-  kill_cleanup(p,(void*)fd,fd_cleanup);
+static void register_fd_cleanups(pool *p, int fd) {
+  register_cleanup(p, (void *)fd, fd_cleanup_cb, fd_cleanup_cb);
 }
 
-int popenf(pool *p, const char *name, int flg, int mode)
-{
+int popenf(pool *p, const char *name, int flags, int mode) {
   int fd;
 
   block_alarms();
-  fd = open(name,flg,mode);
-  if(fd >= 0)
-    note_cleanups_for_fd(p,fd);
+  if ((fd = open(name, flags, mode)) >= 0)
+    register_fd_cleanups(p, fd);
   unblock_alarms();
   return fd;
 }
 
-int pclosef(pool *p, int fd)
-{
+int pclosef(pool *p, int fd) {
   int res;
 
   block_alarms();
   res = close(fd);
-  kill_cleanup(p, (void*)fd, fd_cleanup);
+  unregister_cleanup(p, (void *)fd, fd_cleanup_cb);
   unblock_alarms();
   return res;
 }
@@ -795,59 +770,58 @@ int pclosef(pool *p, int fd)
  * the stream
  */
 
-static void file_cleanup(void *fpv) { fclose((FILE*)fpv); }
-static void file_child_cleanup(void *fpv)
-{ close(fileno((FILE*)fpv)); }
-
-void note_cleanups_for_file(pool *p, FILE *fp)
-{
-  register_cleanup(p,(void*)fp,file_cleanup,file_child_cleanup);
+static void file_cleanup_cb(void *fpv) {
+  fclose((FILE *)fpv);
 }
 
-FILE *pfopen(pool *p, const char *name, const char *mode)
-{
+static void file_child_cleanup_cb(void *fpv) {
+  close(fileno((FILE *) fpv));
+}
+
+void register_file_cleanups(pool *p, FILE *fp) {
+  register_cleanup(p, (void *)fp, file_cleanup_cb, file_child_cleanup_cb);
+}
+
+FILE *pfopen(pool *p, const char *name, const char *mode) {
   FILE *fd = NULL;
-  int baseFlag, desc;
+  int base_flag, desc;
 
   block_alarms();
 
-  if(*mode == 'a') {
-    baseFlag = (*(mode+1) == '+') ? O_RDWR : O_WRONLY;
-    desc = open(name, baseFlag | O_APPEND | O_CREAT,
-                S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
-    if(desc >= 0)
+  if (*mode == 'a') {
+    base_flag = (*(mode+1) == '+') ? O_RDWR : O_WRONLY;
+    desc = open(name, base_flag|O_APPEND|O_CREAT,
+       S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
+    if (desc >= 0)
       fd = fdopen(desc, mode);
-  } else {
-    fd = fopen(name, mode);
-  }
 
-  if(fd)
-    note_cleanups_for_file(p,fd);
+  } else
+    fd = fopen(name, mode);
+
+  if (fd)
+    register_file_cleanups(p, fd);
+
   unblock_alarms();
   return fd;
 }
 
-FILE *pfdopen(pool *p, int fd, const char *mode)
-{
+FILE *pfdopen(pool *p, int fd, const char *mode) {
   FILE *f;
 
   block_alarms();
-  f = fdopen(fd,mode);
-  if(f)
-    note_cleanups_for_file(p,f);
+  if ((f = fdopen(fd, mode)) != NULL)
+    register_file_cleanups(p, f);
 
   unblock_alarms();
   return f;
 }
 
-int pfclose(pool *p, FILE *fd)
-{
+int pfclose(pool *p, FILE *fd) {
   int res;
 
   block_alarms();
   res = fclose(fd);
-  kill_cleanup(p, (void*)fd, file_cleanup);
+  unregister_cleanup(p, (void *) fd, file_cleanup_cb);
   unblock_alarms();
   return res;
 }
-
