@@ -1,7 +1,8 @@
 /*
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
- * Copyright (C) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 2001, 2002 The ProFTPD Project team
  *  
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +26,7 @@
 
 /* Various basic support routines for ProFTPD, used by all modules
  * and not specific to one or another.
- * $Id: support.c,v 1.29 2002/05/21 20:47:23 castaglia Exp $
+ * $Id: support.c,v 1.35 2002/08/14 16:17:35 castaglia Exp $
  */
 
 /* History Log:
@@ -89,6 +90,7 @@ typedef struct _exithandler {
 typedef struct _sched {
   struct _sched *next,*prev;
 
+  pool *pool;
   void (*f)(void*,void*,void*,void*);
   int loops;
   void *a1,*a2,*a3,*a4;
@@ -97,13 +99,12 @@ typedef struct _sched {
 static xaset_t *scheds = NULL;
 static xaset_t *exits = NULL;
 
-/* performs "hard block" of all important signals (as opposed to 
- * block_alarms) */
-static void _block_signals(int block)
-{
+/* Masks/unmasks all important signals (as opposed to * block_alarms)
+ */
+static void mask_signals(unsigned char block) {
   static sigset_t sigset;
 
-  if(block) {
+  if (block) {
     sigemptyset(&sigset);
 
     sigaddset(&sigset,SIGTERM);
@@ -121,18 +122,23 @@ static void _block_signals(int block)
     sigaddset(&sigset,SIGHUP);
 
     sigprocmask(SIG_BLOCK,&sigset,NULL);
-  } else
+
+  } else {
     sigprocmask(SIG_UNBLOCK,&sigset,NULL);
+
+    /* If unmasking, handle any signals that may have been delivered while
+     * masked.
+     */
+    pr_handle_signals();
+  }
 }
 
-void block_signals(void)
-{
-  _block_signals(1);
+void block_signals(void) {
+  mask_signals(TRUE);
 }
 
-void unblock_signals(void)
-{
-  _block_signals(0);
+void unblock_signals(void) {
+  mask_signals(FALSE);
 }
 
 void add_exit_handler(void (*exit)()) {
@@ -173,16 +179,20 @@ void run_exit_handlers(void) {
 void schedule(void (*f)(void*,void*,void*,void*),int nloops,
               void *a1, void *a2, void *a3, void *a4)
 {
-  pool *p;
+  pool *p, *sub_pool;
   sched_t *s;
 
-  if(!scheds) {
+  if (!scheds) {
    p = make_sub_pool(permanent_pool);
-   scheds = xaset_create(p,NULL);
+   scheds = xaset_create(p, NULL);
+
   } else
    p = scheds->mempool;
 
-  s = (sched_t*)pcalloc(p,sizeof(sched_t));
+  sub_pool = make_sub_pool(p);
+
+  s = pcalloc(sub_pool, sizeof(sched_t));
+  s->pool = sub_pool;
   s->f = f;
   s->a1 = a1;
   s->a2 = a2;
@@ -192,11 +202,9 @@ void schedule(void (*f)(void*,void*,void*,void*),int nloops,
   xaset_insert(scheds,(xasetmember_t*)s);
 }
 
-void run_schedule(void)
-{
+void run_schedule(void) {
   sched_t *s,*snext;
 
-  handle_sig_alarm();
   if(!scheds || !scheds->xas_list)
     return;
 
@@ -206,18 +214,11 @@ void run_schedule(void)
     if(s->loops-- <= 0) {
       s->f(s->a1,s->a2,s->a3,s->a4);
       xaset_remove(scheds,(xasetmember_t*)s);
+      destroy_pool(s->pool);
     }
   }
 }
 
-/* Returns TRUE if there is a scheduled function waiting */
-int schedulep(void)
-{
-  handle_sig_alarm();
-  return (scheds && scheds->xas_list);
-}
-
-
 /*
 ** Get the maximum size of a file name (pathname component).
 ** If a directory file descriptor, e.g. the d_fd DIR structure element,
@@ -822,15 +823,14 @@ char *sreplace(pool *p, char *s, ...)
   return pstrdup(p,buf);
 }
 
+#if defined(HAVE_SYS_STATVFS_H) || defined(HAVE_SYS_VFS_H)
 /* Simple multiplication & division doesn't work with very large
  * filesystems (overflows 32 bits).  This code should handle it.
  */
 
-static
-unsigned long _calc_fs(unsigned long blocks, unsigned long bsize)
-{
-  unsigned long bl_lo,bl_hi;
-  unsigned long res_lo,res_hi,tmp;
+static off_t _calc_fs(size_t blocks, size_t bsize) {
+  off_t bl_lo,bl_hi;
+  off_t res_lo,res_hi,tmp;
 
   bl_lo = blocks & 0x0000ffff;
   bl_hi = blocks & 0xffff0000;
@@ -846,9 +846,8 @@ unsigned long _calc_fs(unsigned long blocks, unsigned long bsize)
   return (res_lo >> 10) | (res_hi << 6);
 }
 
-#ifdef HAVE_SYS_STATVFS_H
-unsigned long get_fs_size(char *s)
-{
+# ifdef HAVE_SYS_STATVFS_H
+off_t get_fs_size(char *s) {
   struct statvfs vfs;
 
   if(statvfs(s,&vfs) != 0)
@@ -856,9 +855,8 @@ unsigned long get_fs_size(char *s)
 
   return _calc_fs(vfs.f_bavail,vfs.f_frsize);
 }
-#elif defined(HAVE_SYS_VFS_H)
-unsigned long get_fs_size(char *s)
-{
+# elif defined(HAVE_SYS_VFS_H)
+off_t get_fs_size(char *s) {
   struct statfs vfs;
 
   if(statfs(s,&vfs) != 0)
@@ -866,7 +864,8 @@ unsigned long get_fs_size(char *s)
 
   return _calc_fs(vfs.f_bavail,vfs.f_bsize);
 }
-#endif /* HAVE_SYS_STATVFS/HAVE_SYS_VFS */
+# endif /* no HAVE_SYS_STATVFS/HAVE_SYS_VFS */
+#endif /* no HAVE_SYS_STATVFS/HAVE_SYS_VFS */
 
 /* "safe" strcat, saves room for \0 at end of dest, and refuses to copy
  * more than "n" bytes.
